import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class TestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        # Test covariant return types for pointers to class that contains the called function.
        self.expect_expr("derived.getPtr()", result_type="Derived *")
        self.expect_expr("base_ptr_to_derived->getPtr()", result_type="Base *")
        self.expect_expr("base.getPtr()", result_type="Base *")
        # The same tests with reference types. LLDB drops the reference when it turns the
        # result into a SBValue so check for the underlying type of the result.
        self.expect_expr("derived.getRef()", result_type="Derived")
        self.expect_expr("base_ptr_to_derived->getRef()", result_type="Base")
        self.expect_expr("base.getRef()", result_type="Base")

        # Test covariant return types for pointers to class that does *not* contain the called function.
        self.expect_expr("derived.getOtherPtr()", result_type="OtherDerived *")
        self.expect_expr(
            "base_ptr_to_derived->getOtherPtr()", result_type="OtherBase *"
        )
        self.expect_expr("base.getOtherPtr()", result_type="OtherBase *")
        # The same tests with reference types. LLDB drops the reference when it turns the
        # result into a SBValue so check for the underlying type of the result.
        self.expect_expr("derived.getOtherRef()", result_type="OtherDerived")
        self.expect_expr("base_ptr_to_derived->getOtherRef()", result_type="OtherBase")
        self.expect_expr("base.getOtherRef()", result_type="OtherBase")

        # Test that we call the right function and get the right value back.
        self.expect_expr("derived.getOtherPtr()->value()", result_summary='"derived"')
        self.expect_expr(
            "base_ptr_to_derived->getOtherPtr()->value()", result_summary='"derived"'
        )
        self.expect_expr("base.getOtherPtr()->value()", result_summary='"base"')
        self.expect_expr("derived.getOtherRef().value()", result_summary='"derived"')
        self.expect_expr(
            "base_ptr_to_derived->getOtherRef().value()", result_summary='"derived"'
        )
        self.expect_expr("base.getOtherRef().value()", result_summary='"base"')

        self.expect_expr("referencing_derived.getOther()->get()->a", result_value="42")



"""
Test that we recover gracefully from evaluating a breakpoint condition that crashes.
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestCrashingCondition(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_crashing_condition(self):
        """Test that if a condition crashes we stop and recover cleanly"""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        self.do_crash_condition()

    def do_crash_condition(self):
        """Test that if a condition crashes we stop and recover cleanly"""

        # This function starts a process, "a.out" by default, sets a source
        # breakpoint, runs to it, and returns the thread, process & target.
        # It optionally takes an SBLaunchOption argument if you want to pass
        # arguments or environment variables.
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a start breakpoint here", self.main_source_file
        )

        # Set a breakpoint with a condition that crashes on the next line:
        bad_bkpt = target.BreakpointCreateBySourceRegex(
            "Set the test breakpoint here", self.main_source_file
        )
        self.assertGreater(
            bad_bkpt.GetNumLocations(), 0, "Found locations for our breakpoint"
        )
        bad_bkpt.SetCondition("do_crash()")

        self.runCmd("continue")

        self.assertState(process.state, lldb.eStateStopped)
        self.assertStopReason(thread.stop_reason, lldb.eStopReasonBreakpoint)

        # Now we should be able to continue to the real crash:
        error = lldb.SBError()
        self.runCmd("continue")

        # We should have crashed.
        self.assertState(process.state, lldb.eStateStopped)
        # We don't actually know what stop reason a given system will
        # report - it could be eStopReasonException or eStopReasonSignal
        is_crash = (
            thread.stop_reason == lldb.eStopReasonException
            or thread.stop_reason == lldb.eStopReasonSignal
        )
        self.assertIn(thread.stop_reason, [lldb.eStopReasonException, lldb.eStopReasonSignal], "Ran to the actual crash")



# RUN: %PYTHON %s | FileCheck %s

import gc
from mlir.ir import *


def run(f):
    print("\nTEST:", f.__name__)
    f()
    gc.collect()
    assert Context._get_live_count() == 0
    return f


# CHECK-LABEL: TEST: testIntegerSetCapsule
@run
def testIntegerSetCapsule():
    with Context() as ctx:
        is1 = IntegerSet.get_empty(1, 1, ctx)
    capsule = is1._CAPIPtr
    # CHECK: mlir.ir.IntegerSet._CAPIPtr
    print(capsule)
    is2 = IntegerSet._CAPICreate(capsule)
    assert is1 == is2
    assert is2.context is ctx


# CHECK-LABEL: TEST: testIntegerSetGet
@run
def testIntegerSetGet():
    with Context():
        d0 = AffineDimExpr.get(0)
        d1 = AffineDimExpr.get(1)
        s0 = AffineSymbolExpr.get(0)
        c42 = AffineConstantExpr.get(42)

        # CHECK: (d0, d1)[s0] : (d0 - d1 == 0, s0 - 42 >= 0)
        set0 = IntegerSet.get(2, 1, [d0 - d1, s0 - c42], [True, False])
        print(set0)

        # CHECK: (d0)[s0] : (1 == 0)
        set1 = IntegerSet.get_empty(1, 1)
        print(set1)

        # CHECK: (d0)[s0, s1] : (d0 - s1 == 0, s0 - 42 >= 0)
        set2 = set0.get_replaced([d0, AffineSymbolExpr.get(1)], [s0], 1, 2)
        print(set2)

        try:
            IntegerSet.get(2, 1, [], [])
        except ValueError as e:
            # CHECK: Expected non-empty list of constraints
            print(e)

        try:
            IntegerSet.get(2, 1, [d0 - d1], [True, False])
        except ValueError as e:
            # CHECK: Expected the number of constraints to match that of equality flags
            print(e)

        try:
            IntegerSet.get(2, 1, [0], [True])
        except RuntimeError as e:
            # CHECK: Invalid expression when attempting to create an IntegerSet
            print(e)

        try:
            IntegerSet.get(2, 1, [None], [True])
        except RuntimeError as e:
            # CHECK: Invalid expression (None?) when attempting to create an IntegerSet
            print(e)

        try:
            set0.get_replaced([d0], [s0], 1, 1)
        except ValueError as e:
            # CHECK: Expected the number of dimension replacement expressions to match that of dimensions
            print(e)

        try:
            set0.get_replaced([d0, d1], [s0, s0], 1, 1)
        except ValueError as e:
            # CHECK: Expected the number of symbol replacement expressions to match that of symbols
            print(e)

        try:
            set0.get_replaced([d0, 1], [s0], 1, 1)
        except RuntimeError as e:
            # CHECK: Invalid expression when attempting to create an IntegerSet by replacing dimensions
            print(e)

        try:
            set0.get_replaced([d0, d1], [None], 1, 1)
        except RuntimeError as e:
            # CHECK: Invalid expression (None?) when attempting to create an IntegerSet by replacing symbols
            print(e)


# CHECK-LABEL: TEST: testIntegerSetProperties
@run
def testIntegerSetProperties():
    with Context():
        d0 = AffineDimExpr.get(0)
        d1 = AffineDimExpr.get(1)
        s0 = AffineSymbolExpr.get(0)
        c42 = AffineConstantExpr.get(42)

        set0 = IntegerSet.get(2, 1, [d0 - d1, s0 - c42, s0 - d0], [True, False, False])
        # CHECK: 2
        print(set0.n_dims)
        # CHECK: 1
        print(set0.n_symbols)
        # CHECK: 3
        print(set0.n_inputs)
        # CHECK: 1
        print(set0.n_equalities)
        # CHECK: 2
        print(set0.n_inequalities)

        # CHECK: 3
        print(len(set0.constraints))

        # CHECK-DAG: d0 - d1 == 0
        # CHECK-DAG: s0 - 42 >= 0
        # CHECK-DAG: -d0 + s0 >= 0
        for cstr in set0.constraints:
            print(cstr.expr, end="")
            print(" == 0" if cstr.is_eq else " >= 0")


# TODO-LABEL: TEST: testHash
@run
def testHash():
    with Context():
        d0 = AffineDimExpr.get(0)
        d1 = AffineDimExpr.get(1)
        set = IntegerSet.get(2, 0, [d0 + d1], [True])

        assert hash(set) == hash(IntegerSet.get(2, 0, [d0 + d1], [True]))

        dictionary = dict()
        dictionary[set] = 42
        assert set in dictionary



"""
Test LLDB can set and hit watchpoints on tagged addresses
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestWatchTaggedAddresses(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

        # Skip this test if not running on AArch64 target that supports PAC
        if not self.isAArch64PAuth():
            self.skipTest("Target must support pointer authentication.")

        # Set source filename.
        self.source = "main.c"

        # Invoke the default build rule.
        self.build()

        # Get the path of the executable
        exe = self.getBuildArtifact("a.out")

        # Create a target by the debugger.
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_watch_hit_tagged_ptr_access(self):
        """
        Test that LLDB hits watchpoint installed on an untagged address with
        memory access by a tagged pointer.
        """

        # Add a breakpoint to set a watchpoint when stopped on the breakpoint.
        lldbutil.run_break_set_by_symbol(self, "main")

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # We should be stopped due to the breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # Set the watchpoint variable declaration line number.
        self.decl = line_number(self.source, "// Watchpoint variable declaration.")

        # Now let's set a watchpoint on 'global_var'.
        self.expect(
            "watchpoint set variable global_var",
            WATCHPOINT_CREATED,
            substrs=[
                "Watchpoint created",
                "size = 4",
                "type = m",
                "%s:%d" % (self.source, self.decl),
            ],
        )

        self.verify_watch_hits()

    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_watch_set_on_tagged_ptr(self):
        """Test that LLDB can install and hit watchpoint on a tagged address"""

        # Find the line number to break inside main().
        self.line = line_number(self.source, "// Set break point at this line.")

        # Add a breakpoint to set a watchpoint when stopped on the breakpoint.
        lldbutil.run_break_set_by_file_and_line(
            self, None, self.line, num_expected_locations=1
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # We should be stopped due to the breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # Now let's set a expression watchpoint on 'tagged_ptr'.
        self.expect(
            "watchpoint set expression -s 4 -- tagged_ptr",
            WATCHPOINT_CREATED,
            substrs=["Watchpoint created", "size = 4", "type = m"],
        )

        self.verify_watch_hits()

    def verify_watch_hits(self):
        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should be 0 initially.
        self.expect(
            "watchpoint list -v",
            substrs=["Number of supported hardware watchpoints:", "hit_count = 0"],
        )

        self.runCmd("process continue")

        # We should be stopped again due to the watchpoint (read_write type).
        # The stop reason of the thread should be watchpoint.
        self.expect(
            "thread backtrace",
            STOPPED_DUE_TO_WATCHPOINT,
            substrs=["stop reason = watchpoint"],
        )

        self.runCmd("process continue")

        # We should be stopped again due to the watchpoint (read_write type).
        # The stop reason of the thread should be watchpoint.
        self.expect(
            "thread backtrace",
            STOPPED_DUE_TO_WATCHPOINT,
            substrs=["stop reason = watchpoint"],
        )

        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should now be 2.
        self.expect("watchpoint list -v", substrs=["hit_count = 2"])

        # On some hardware, during __do_global_dtors_aux a flag is set near
        # the global which can trigger the watchpoint. So we must remove it.
        self.runCmd("watchpoint delete 1")
        self.runCmd("process continue")

        # There should be no more watchpoint hit and the process status should
        # be 'exited'.
        self.expect("process status", substrs=["exited"])



"""
Test basic std::vector functionality.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestBasicVector(TestBase):
    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    @skipIf(bugnumber="rdar://100741983")
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.import-std-module true")

        vector_type = "std::vector<int>"
        size_type = "size_type"
        value_type = "value_type"
        iterator = "iterator"
        # LLDB's formatter provides us with a artificial 'item' member.
        iterator_children = [ValueCheck(name="item")]
        riterator = "reverse_iterator"
        riterator_children = [
            ValueCheck(),  # Deprecated __t_ member; no need to check
            ValueCheck(name="current"),
        ]

        self.expect_expr(
            "a",
            result_type=vector_type,
            result_children=[
                ValueCheck(value="3"),
                ValueCheck(value="1"),
                ValueCheck(value="2"),
            ],
        )
        self.expect_expr("a.size()", result_type=size_type, result_value="3")
        front = self.expect_expr("a.front()", result_type=value_type, result_value="3")
        self.expect_expr("a[1]", result_type=value_type, result_value="1")
        self.expect_expr("a.back()", result_type=value_type, result_value="2")

        self.expect("expr std::sort(a.begin(), a.end())")
        self.expect_expr("a.front()", result_type=value_type, result_value="1")
        self.expect_expr("a[1]", result_type=value_type, result_value="2")
        self.expect_expr("a.back()", result_type=value_type, result_value="3")

        self.expect("expr std::reverse(a.begin(), a.end())")
        self.expect_expr("a.front()", result_type=value_type, result_value="3")
        self.expect_expr("a[1]", result_type=value_type, result_value="2")
        self.expect_expr("a.back()", result_type=value_type, result_value="1")

        self.expect_expr(
            "a.begin()", result_type=iterator, result_children=iterator_children
        )
        self.expect_expr(
            "a.rbegin()", result_type=riterator, result_children=riterator_children
        )

        self.expect_expr("*a.begin()", result_type="int", result_value="3")
        self.expect_expr("*a.rbegin()", result_type="int", result_value="1")

        self.expect("expr a.pop_back()")
        self.expect_expr("a.back()", result_type=value_type, result_value="2")
        self.expect_expr("a.size()", result_type=size_type, result_value="2")

        self.expect_expr("a.at(0)", result_type=value_type, result_value="3")

        self.expect("expr a.push_back(4)")
        self.expect_expr("a.back()", result_type=value_type, result_value="4")
        self.expect_expr("a.size()", result_type=size_type, result_value="3")

        self.expect("expr a.emplace_back(5)")
        self.expect_expr("a.back()", result_type=value_type, result_value="5")
        self.expect_expr("a.size()", result_type=size_type, result_value="4")

        self.expect_expr(
            "a",
            result_children=[
                ValueCheck(value="3"),
                ValueCheck(value="2"),
                ValueCheck(value="4"),
                ValueCheck(value="5"),
            ],
        )

        # Test that the typedef'd vector type can be substituted.
        self.expect("expr b.emplace_back(6)")
        self.expect_expr(
            "b",
            result_type="vector_long",
            result_children=[
                ValueCheck(value="3"),
                ValueCheck(value="1"),
                ValueCheck(value="2"),
                ValueCheck(value="6"),
            ],
        )



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
import numpy as np
import mlir.dialects.func as func
import mlir.dialects.shape as shape


def run(f):
    print("\nTEST:", f.__name__)
    f()
    return f


# CHECK-LABEL: TEST: testConstShape
@run
def testConstShape():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        f32 = F32Type.get()
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(
                RankedTensorType.get((12, ShapedType.get_dynamic_size()), f32)
            )
            def const_shape_tensor(arg):
                shape.ConstWitnessOp(False)
                shape.ConstSizeOp(30)
                shape.ConstSizeOp(IntegerAttr.get(IndexType.get(), 40))
                x = shape.ConstShapeOp([1, 2])
                shape.MeetOp(x, x, error="impossible")
                return shape.ConstShapeOp(
                    DenseElementsAttr.get(
                        np.array([3, 4], dtype=np.int64), type=IndexType.get()
                    )
                )

        # CHECK-LABEL: func @const_shape_tensor(%arg0: tensor<12x?xf32>)
        # CHECK-DAG: shape.const_witness false
        # CHECK-DAG: shape.const_size 30
        # CHECK-DAG: shape.const_size 40
        # CHECK-DAG: shape.const_shape [1, 2] : tensor<2xindex>
        # CHECK-DAG: shape.const_shape [3, 4] : tensor<2xindex>
        print(module)



# RUN: %python %s 2>&1 | FileCheck %s
import sys

# CHECK: Testing mlir_standalone package
print("Testing mlir_standalone package", file=sys.stderr)

import mlir_standalone.ir
from mlir_standalone.dialects import standalone_nanobind as standalone_d

with mlir_standalone.ir.Context():
    standalone_d.register_dialects()
    standalone_module = mlir_standalone.ir.Module.parse(
        """
    %0 = arith.constant 2 : i32
    %1 = standalone.foo %0 : i32
    """
    )
    # CHECK: %[[C2:.*]] = arith.constant 2 : i32
    # CHECK: standalone.foo %[[C2]] : i32
    print(str(standalone_module), file=sys.stderr)

    custom_type = standalone_d.CustomType.get("foo")
    # CHECK: !standalone.custom<"foo">
    print(custom_type, file=sys.stderr)

    # CHECK: this is a fp16 type
    standalone_d.print_fp_type(mlir_standalone.ir.F16Type.get(), sys.stderr)
    # CHECK: this is a fp32 type
    standalone_d.print_fp_type(mlir_standalone.ir.F32Type.get(), sys.stderr)
    # CHECK: this is a fp64 type
    standalone_d.print_fp_type(mlir_standalone.ir.F64Type.get(), sys.stderr)


# CHECK: Testing mlir package
print("Testing mlir package", file=sys.stderr)

from mlir.ir import *

# CHECK-NOT: RuntimeWarning: nanobind: type '{{.*}}' was already registered!



import lldb

num_hits = 0


def watchpoint_command(frame, wp, dict):
    global num_hits
    if num_hits == 0:
        print("I stopped the first time")
        frame.EvaluateExpression("cookie = 888")
        num_hits += 1
        return True
    if num_hits == 1:
        print("I stopped the second time, but with no return")
        frame.EvaluateExpression("cookie = 666")
        num_hits += 1
        return True
    else:
        print("I stopped the %d time" % (num_hits))
        frame.EvaluateExpression("cookie = 999")
        return False  # This cause the process to continue.



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
import mlir.dialects.emitc as emitc


def run(f):
    print("\nTEST:", f.__name__)
    with Context() as ctx, Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            f(ctx)
        print(module)


# CHECK-LABEL: TEST: testConstantOp
@run
def testConstantOp(ctx):
    i32 = IntegerType.get_signless(32)
    a = emitc.ConstantOp(result=i32, value=IntegerAttr.get(i32, 42))
    # CHECK: %{{.*}} = "emitc.constant"() <{value = 42 : i32}> : () -> i32


# CHECK-LABEL: TEST: testAddOp
@run
def testAddOp(ctx):
    i32 = IntegerType.get_signless(32)
    lhs = emitc.ConstantOp(result=i32, value=IntegerAttr.get(i32, 0))
    rhs = emitc.ConstantOp(result=i32, value=IntegerAttr.get(i32, 0))
    a = emitc.AddOp(i32, lhs, rhs)
    # CHECK: %{{.*}} = emitc.add %{{.*}}, %{{.*}} : (i32, i32) -> i32



"""
Tests debugserver support for MultiMemRead.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


@skipUnlessDarwin
@skipIfOutOfTreeDebugserver
class TestCase(TestBase):
    def send_process_packet(self, packet_str):
        self.runCmd(f"proc plugin packet send {packet_str}", check=False)
        # The output is of the form:
        #  packet: <packet_str>
        #  response: <response>
        reply = self.res.GetOutput().split("\n")
        packet = reply[0].strip()
        response = reply[1].strip()

        self.assertTrue(packet.startswith("packet: "))
        self.assertTrue(response.startswith("response: "))
        return response[len("response: ") :]

    def check_invalid_packet(self, packet_str):
        reply = self.send_process_packet("packet_str")
        self.assertEqual(reply, "E03")

    def test_packets(self):
        self.build()
        source_file = lldb.SBFileSpec("main.c")
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "break here", source_file
        )

        reply = self.send_process_packet("qSupported")
        self.assertIn("MultiMemRead+", reply)

        mem_address_var = thread.frames[0].FindVariable("memory")
        self.assertTrue(mem_address_var)
        mem_address = mem_address_var.GetValueAsUnsigned() + 42

        # no ":"
        self.check_invalid_packet("MultiMemRead")
        # missing ranges
        self.check_invalid_packet("MultiMemRead:")
        # needs at least one range
        self.check_invalid_packet("MultiMemRead:ranges:")
        # needs at least one range
        self.check_invalid_packet("MultiMemRead:ranges:,")
        # a range is a pair of numbers
        self.check_invalid_packet("MultiMemRead:ranges:10")
        # a range is a pair of numbers
        self.check_invalid_packet("MultiMemRead:ranges:10,")
        # range list must end with ;
        self.check_invalid_packet("MultiMemRead:ranges:10,2")
        self.check_invalid_packet("MultiMemRead:ranges:10,2,")
        self.check_invalid_packet("MultiMemRead:ranges:10,2,3")
        # ranges are pairs of numbers.
        self.check_invalid_packet("MultiMemRead:ranges:10,2,3;")
        # unrecognized field
        self.check_invalid_packet("MultiMemRead:ranges:10,2;blah:;")
        # unrecognized field
        self.check_invalid_packet("MultiMemRead:blah:;ranges:10,2;")

        # Zero-length reads are ok.
        reply = self.send_process_packet("MultiMemRead:ranges:0,0;")
        self.assertEqual(reply, "0;")

        # Debugserver is permissive with trailing commas.
        reply = self.send_process_packet("MultiMemRead:ranges:10,2,;")
        self.assertEqual(reply, "0;")
        reply = self.send_process_packet(f"MultiMemRead:ranges:{mem_address:x},2,;")
        self.assertEqual(reply, "2;ab")

        reply = self.send_process_packet("MultiMemRead:ranges:10,2;")
        self.assertEqual(reply, "0;")
        reply = self.send_process_packet(f"MultiMemRead:ranges:{mem_address:x},0;")
        self.assertEqual(reply, "0;")
        reply = self.send_process_packet(f"MultiMemRead:ranges:{mem_address:x},2;")
        self.assertEqual(reply, "2;ab")
        reply = self.send_process_packet(
            f"MultiMemRead:ranges:{mem_address:x},2,{mem_address+2:x},4;"
        )
        self.assertEqual(reply, "2,4;abcdef")
        reply = self.send_process_packet(
            f"MultiMemRead:ranges:{mem_address:x},2,{mem_address+2:x},4,{mem_address+6:x},8;"
        )
        self.assertEqual(reply, "2,4,8;abcdefghijklmn")

        # Test zero length in the middle.
        reply = self.send_process_packet(
            f"MultiMemRead:ranges:{mem_address:x},2,{mem_address+2:x},0,{mem_address+6:x},8;"
        )
        self.assertEqual(reply, "2,0,8;abghijklmn")
        # Test zero length in the end.
        reply = self.send_process_packet(
            f"MultiMemRead:ranges:{mem_address:x},2,{mem_address+2:x},4,{mem_address+6:x},0;"
        )
        self.assertEqual(reply, "2,4,0;abcdef")



import re

import gdbremote_testcase
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class TestPartialResume(gdbremote_testcase.GdbRemoteTestCaseBase):
    THREAD_MATCH_RE = re.compile(r"thread ([0-9a-f]+) running")

    def start_vCont_run_subset_of_threads_test(self):
        self.build()
        self.set_inferior_startup_launch()

        procs = self.prep_debug_monitor_and_inferior(inferior_args=["3"])
        # grab the main thread id
        self.add_threadinfo_collection_packets()
        main_thread = self.parse_threadinfo_packets(self.expect_gdbremote_sequence())
        self.assertEqual(len(main_thread), 1)
        self.reset_test_sequence()

        # run until threads start, then grab full thread list
        self.test_sequence.add_log_lines(
            [
                "read packet: $c#63",
                {"direction": "send", "regex": "[$]T.*;reason:signal.*"},
            ],
            True,
        )
        self.add_threadinfo_collection_packets()

        all_threads = self.parse_threadinfo_packets(self.expect_gdbremote_sequence())
        self.assertEqual(len(all_threads), 4)
        self.assertIn(main_thread[0], all_threads)
        self.reset_test_sequence()

        all_subthreads = set(all_threads) - set(main_thread)
        self.assertEqual(len(all_subthreads), 3)

        return (main_thread[0], list(all_subthreads))

    def continue_and_get_threads_running(self, main_thread, vCont_req):
        self.test_sequence.add_log_lines(
            [
                "read packet: $vCont;c:{:x};{}#00".format(main_thread, vCont_req),
                "send packet: $W00#00",
            ],
            True,
        )
        exp = self.expect_gdbremote_sequence()
        self.reset_test_sequence()
        found = set()
        for line in exp["O_content"].decode().splitlines():
            m = self.THREAD_MATCH_RE.match(line)
            if m is not None:
                found.add(int(m.group(1), 16))
        return found

    @skipIfWindows
    @add_test_categories(["llgs"])
    def test_vCont_cxcx(self):
        main_thread, all_subthreads_list = self.start_vCont_run_subset_of_threads_test()
        # resume two threads explicitly, stop the third one implicitly
        self.assertEqual(
            self.continue_and_get_threads_running(
                main_thread, "c:{:x};c:{:x}".format(*all_subthreads_list[:2])
            ),
            set(all_subthreads_list[:2]),
        )

    @skipIfWindows
    @add_test_categories(["llgs"])
    def test_vCont_cxcxt(self):
        main_thread, all_subthreads_list = self.start_vCont_run_subset_of_threads_test()
        # resume two threads explicitly, stop others explicitly
        self.assertEqual(
            self.continue_and_get_threads_running(
                main_thread, "c:{:x};c:{:x};t".format(*all_subthreads_list[:2])
            ),
            set(all_subthreads_list[:2]),
        )



"""
Test the 'gui' shortcuts 's','n','f','u','d' (step in, step over, step out, up, down)
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.lldbpexpect import PExpectTest


class TestGuiBasicDebugCommandTest(PExpectTest):
    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    @skipIf(bugnumber="llvm.org/pr51833")
    @skipIfCursesSupportMissing
    def test_gui(self):
        self.build()

        self.launch(executable=self.getBuildArtifact("a.out"), dimensions=(100, 500))
        self.expect(
            'br set -f main.c -p "// Break here"', substrs=["Breakpoint 1", "address ="]
        )
        self.expect("run", substrs=["stop reason ="])

        escape_key = chr(27).encode()

        # Start the GUI.
        self.child.sendline("gui")

        # Simulate a simple debugging session.
        self.child.send("s")  # step
        self.child.expect("return 1; // In function[^\r\n]+<<< Thread 1: step in")
        self.child.send("u")  # up
        self.child.expect_exact("func();    // Break here")
        self.child.send("d")  # down
        self.child.expect_exact("return 1; // In function")
        self.child.send("f")  # finish
        self.child.expect("<<< Thread 1: step out")
        self.child.send("s")  # move onto the second one
        self.child.expect("<<< Thread 1: step in")
        self.child.send("n")  # step over
        self.child.expect("// Dummy command 1[^\r\n]+<<< Thread 1: step over")
        self.child.send("n")

        # Test that 'up' + 'step out' steps out of the selected function.
        self.child.send("s")  # move into func_up()
        self.child.expect("// In func_up")
        self.child.send("s")  # move into func_down()
        self.child.expect("// In func_down")
        self.child.send("u")  # up
        self.child.expect("// In func_up")
        self.child.send("f")  # finish
        self.child.expect("// Dummy command 2[^\r\n]+<<< Thread 1: step out")
        self.child.send("n")

        # Press escape to quit the gui
        self.child.send(escape_key)

        self.expect_prompt()
        self.quit()



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestqOffsets(GDBRemoteTestBase):
    class Responder(MockGDBServerResponder):
        def qOffsets(self):
            return "Text=470000;Data=470000"

    def test(self):
        self.server.responder = TestqOffsets.Responder()
        # This ensures that we do not pick up any binaries on the host.
        self.runCmd("platform select remote-linux")
        target = self.createTarget("qOffsets.yaml")
        text = target.modules[0].FindSection(".text")
        self.assertEqual(text.GetLoadAddress(target), lldb.LLDB_INVALID_ADDRESS)

        process = self.connect(target)
        self.assertEqual(text.GetLoadAddress(target), 0x471000)



"""
Test the lldb Python SBFormat API.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class FormatAPITestCase(TestBase):
    def test_format(self):
        format = lldb.SBFormat()
        self.assertFalse(format)

        error = lldb.SBError()
        format = lldb.SBFormat("${bad}", error)
        self.assertIn("invalid top level item 'bad'", error.GetCString())
        self.assertFalse(format)  # We expect an invalid object back if we have an error
        self.assertTrue(error.Fail())

        format = lldb.SBFormat("${frame.index}", error)
        self.assertIsNone(error.GetCString())
        self.assertTrue(format)
        self.assertTrue(error.Success())



"""
Test thread stepping features in combination with frame select.
"""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class ThreadSteppingTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to of function 'c'.
        self.line1 = line_number(
            "main.c", '// Find the line number of function "c" here.'
        )
        self.line2 = line_number(
            "main.c", '// frame select 2, thread step-out while stopped at "c(1)"'
        )
        self.line3 = line_number("main.c", '// thread step-out while stopped at "c(2)"')
        self.line4 = line_number(
            "main.c", '// frame select 1, thread step-out while stopped at "c(3)"'
        )

    def test_step_out_with_run_command(self):
        """Exercise thread step-out and frame select followed by thread step-out."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Create a breakpoint inside function 'c'.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line1, num_expected_locations=1, loc_exact=True
        )

        # Now run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # The process should be stopped at this point.
        self.expect("process status", PROCESS_STOPPED, patterns=["Process .* stopped"])

        # The frame #0 should correspond to main.c:32, the executable statement
        # in function name 'c'.  And frame #3 should point to main.c:37.
        self.expect(
            "thread backtrace",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint"],
            patterns=[
                "frame #0.*main.c:%d" % self.line1,
                "frame #3.*main.c:%d" % self.line2,
            ],
        )

        # We want to move the pc to frame #3.  This can be accomplished by
        # 'frame select 2', followed by 'thread step-out'.
        self.runCmd("frame select 2")
        self.runCmd("thread step-out")
        self.expect(
            "thread backtrace",
            STEP_OUT_SUCCEEDED,
            substrs=["stop reason = step out"],
            patterns=["frame #0.*main.c:%d" % self.line2],
        )

        # Let's move on to a single step-out case.
        self.runCmd("process continue")

        # The process should be stopped at this point.
        self.expect("process status", PROCESS_STOPPED, patterns=["Process .* stopped"])
        self.runCmd("thread step-out")
        self.expect(
            "thread backtrace",
            STEP_OUT_SUCCEEDED,
            substrs=["stop reason = step out"],
            patterns=["frame #0.*main.c:%d" % self.line3],
        )

        # Do another frame selct, followed by thread step-out.
        self.runCmd("process continue")

        # The process should be stopped at this point.
        self.expect("process status", PROCESS_STOPPED, patterns=["Process .* stopped"])
        self.runCmd("frame select 1")
        self.runCmd("thread step-out")
        self.expect(
            "thread backtrace",
            STEP_OUT_SUCCEEDED,
            substrs=["stop reason = step out"],
            patterns=["frame #0.*main.c:%d" % self.line4],
        )



"""
Test we can understand various layouts of the libc++'s std::unique_ptr
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import functools


class LibcxxUniquePtrDataFormatterSimulatorTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False
    NO_DEBUG_INFO_TESTCASE = True

    def _run_test(self, defines):
        cxxflags_extras = " ".join(["-D%s" % d for d in defines])
        self.build(dictionary=dict(CXXFLAGS_EXTRAS=cxxflags_extras))
        lldbutil.run_to_source_breakpoint(
            self, "Break here", lldb.SBFileSpec("main.cpp")
        )
        self.expect("frame variable var_up", substrs=["pointer ="])
        self.expect("frame variable var_up", substrs=["deleter ="], matching=False)
        self.expect(
            "frame variable var_with_deleter_up", substrs=["pointer =", "deleter ="]
        )


for r in range(5):
    name = "test_r%d" % r
    defines = ["COMPRESSED_PAIR_REV=%d" % r]

    @functools.wraps(LibcxxUniquePtrDataFormatterSimulatorTestCase._run_test)
    def test_method(self, defines=defines):
        LibcxxUniquePtrDataFormatterSimulatorTestCase._run_test(self, defines)

    setattr(LibcxxUniquePtrDataFormatterSimulatorTestCase, name, test_method)



import os, struct, signal

from typing import Any, Dict

import lldb
from lldb.plugins.scripted_process import ScriptedProcess
from lldb.plugins.scripted_process import ScriptedThread


class InvalidScriptedProcess(ScriptedProcess):
    def __init__(self, exe_ctx: lldb.SBExecutionContext, args: lldb.SBStructuredData):
        super().__init__(exe_ctx, args)
        self.threads[0] = InvalidScriptedThread(self, None)

    def read_memory_at_address(
        self, addr: int, size: int, error: lldb.SBError
    ) -> lldb.SBData:
        error.SetErrorString("This is an invalid scripted process!")
        return lldb.SBData()

    def get_loaded_images(self):
        return self.loaded_images

    def get_process_id(self) -> int:
        return 666

    def should_stop(self) -> bool:
        return True

    def is_alive(self) -> bool:
        return True

    def get_scripted_thread_plugin(self):
        return InvalidScriptedThread.__module__ + "." + InvalidScriptedThread.__name__


class InvalidScriptedThread(ScriptedThread):
    def __init__(self, process, args):
        super().__init__(process, args)

    def get_thread_id(self) -> int:
        return 0x19

    def get_name(self) -> str:
        return InvalidScriptedThread.__name__ + ".thread-1"

    def get_state(self) -> int:
        return lldb.eStateInvalid

    def get_stop_reason(self) -> Dict[str, Any]:
        return {"type": lldb.eStopReasonSignal, "data": {"signal": signal.SIGTRAP}}

    def get_register_context(self) -> str:
        return None


def __lldb_init_module(debugger, dict):
    if not "SKIP_SCRIPTED_PROCESS_LAUNCH" in os.environ:
        debugger.HandleCommand(
            "process launch -C %s.%s" % (__name__, InvalidScriptedProcess.__name__)
        )
    else:
        print(
            "Name of the class that will manage the scripted process: '%s.%s'"
            % (__name__, InvalidScriptedProcess.__name__)
        )



"""
Verify that LLDB doesn't crash during expression evaluation.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ExprCrashTestCase(TestBase):
    def test_pr52257(self):
        self.build()
        self.createTestTarget()
        self.expect_expr(
            "b", result_type="B", result_children=[ValueCheck(name="tag_set_")]
        )



# RUN: %PYTHON %s | FileCheck %s

import gc
from mlir.ir import *


def run(f):
    print("\nTEST:", f.__name__)
    f()
    gc.collect()
    assert Context._get_live_count() == 0
    return f


# CHECK-LABEL: TEST: testAffineExprCapsule
@run
def testAffineExprCapsule():
    with Context() as ctx:
        affine_expr = AffineExpr.get_constant(42)

    affine_expr_capsule = affine_expr._CAPIPtr
    # CHECK: capsule object
    # CHECK: mlir.ir.AffineExpr._CAPIPtr
    print(affine_expr_capsule)

    affine_expr_2 = AffineExpr._CAPICreate(affine_expr_capsule)
    assert affine_expr == affine_expr_2
    assert affine_expr_2.context == ctx


# CHECK-LABEL: TEST: testAffineExprEq
@run
def testAffineExprEq():
    with Context():
        a1 = AffineExpr.get_constant(42)
        a2 = AffineExpr.get_constant(42)
        a3 = AffineExpr.get_constant(43)
        # CHECK: True
        print(a1 == a1)
        # CHECK: True
        print(a1 == a2)
        # CHECK: False
        print(a1 == a3)
        # CHECK: False
        print(a1 is None)
        # CHECK: False
        print(a1 == "foo")


# CHECK-LABEL: TEST: testAffineExprContext
@run
def testAffineExprContext():
    with Context():
        a1 = AffineExpr.get_constant(42)
    with Context():
        a2 = AffineExpr.get_constant(42)

    # CHECK: False
    print(a1 == a2)


run(testAffineExprContext)


# CHECK-LABEL: TEST: testAffineExprConstant
@run
def testAffineExprConstant():
    with Context():
        a1 = AffineExpr.get_constant(42)
        # CHECK: 42
        print(a1.value)
        # CHECK: 42
        print(a1)

        a2 = AffineConstantExpr.get(42)
        # CHECK: 42
        print(a2.value)
        # CHECK: 42
        print(a2)

        assert a1 == a2


# CHECK-LABEL: TEST: testAffineExprDim
@run
def testAffineExprDim():
    with Context():
        d1 = AffineExpr.get_dim(1)
        d11 = AffineDimExpr.get(1)
        d2 = AffineDimExpr.get(2)

        # CHECK: 1
        print(d1.position)
        # CHECK: d1
        print(d1)

        # CHECK: 2
        print(d2.position)
        # CHECK: d2
        print(d2)

        assert d1 == d11
        assert d1 != d2


# CHECK-LABEL: TEST: testAffineExprSymbol
@run
def testAffineExprSymbol():
    with Context():
        s1 = AffineExpr.get_symbol(1)
        s11 = AffineSymbolExpr.get(1)
        s2 = AffineSymbolExpr.get(2)

        # CHECK: 1
        print(s1.position)
        # CHECK: s1
        print(s1)

        # CHECK: 2
        print(s2.position)
        # CHECK: s2
        print(s2)

        assert s1 == s11
        assert s1 != s2


# CHECK-LABEL: TEST: testAffineAddExpr
@run
def testAffineAddExpr():
    with Context():
        d1 = AffineDimExpr.get(1)
        d2 = AffineDimExpr.get(2)
        d12 = AffineExpr.get_add(d1, d2)
        # CHECK: d1 + d2
        print(d12)

        d12op = d1 + d2
        # CHECK: d1 + d2
        print(d12op)

        d1cst_op = d1 + 2
        # CHECK: d1 + 2
        print(d1cst_op)

        d1cst_op2 = 2 + d1
        # CHECK: d1 + 2
        print(d1cst_op2)

        assert d12 == d12op
        assert d12.lhs == d1
        assert d12.rhs == d2


# CHECK-LABEL: TEST: testAffineMulExpr
@run
def testAffineMulExpr():
    with Context():
        d1 = AffineDimExpr.get(1)
        c2 = AffineConstantExpr.get(2)
        expr = AffineExpr.get_mul(d1, c2)
        # CHECK: d1 * 2
        print(expr)

        # CHECK: d1 * 2
        op = d1 * c2
        print(op)

        # CHECK: d1 * 2
        op_cst = d1 * 2
        print(op_cst)

        # CHECK: d1 * 2
        op_cst2 = 2 * d1
        print(op_cst2)

        assert expr == op
        assert expr == op_cst
        assert expr.lhs == d1
        assert expr.rhs == c2


# CHECK-LABEL: TEST: testAffineModExpr
@run
def testAffineModExpr():
    with Context():
        d1 = AffineDimExpr.get(1)
        c2 = AffineConstantExpr.get(2)
        expr = AffineExpr.get_mod(d1, c2)
        # CHECK: d1 mod 2
        print(expr)

        # CHECK: d1 mod 2
        op = d1 % c2
        print(op)

        # CHECK: d1 mod 2
        op_cst = d1 % 2
        print(op_cst)

        # CHECK: 2 mod d1
        print(2 % d1)

        assert expr == op
        assert expr == op_cst
        assert expr.lhs == d1
        assert expr.rhs == c2

        expr2 = AffineExpr.get_mod(c2, d1)
        expr3 = AffineExpr.get_mod(2, d1)
        expr4 = AffineExpr.get_mod(d1, 2)

        # CHECK: 2 mod d1
        print(expr2)
        # CHECK: 2 mod d1
        print(expr3)
        # CHECK: d1 mod 2
        print(expr4)

        assert expr2 == expr3
        assert expr4 == expr


# CHECK-LABEL: TEST: testAffineFloorDivExpr
@run
def testAffineFloorDivExpr():
    with Context():
        d1 = AffineDimExpr.get(1)
        c2 = AffineConstantExpr.get(2)
        expr = AffineExpr.get_floor_div(d1, c2)
        # CHECK: d1 floordiv 2
        print(expr)

        assert expr.lhs == d1
        assert expr.rhs == c2

        expr2 = AffineExpr.get_floor_div(c2, d1)
        expr3 = AffineExpr.get_floor_div(2, d1)
        expr4 = AffineExpr.get_floor_div(d1, 2)

        # CHECK: 2 floordiv d1
        print(expr2)
        # CHECK: 2 floordiv d1
        print(expr3)
        # CHECK: d1 floordiv 2
        print(expr4)

        assert expr2 == expr3
        assert expr4 == expr


# CHECK-LABEL: TEST: testAffineCeilDivExpr
@run
def testAffineCeilDivExpr():
    with Context():
        d1 = AffineDimExpr.get(1)
        c2 = AffineConstantExpr.get(2)
        expr = AffineExpr.get_ceil_div(d1, c2)
        # CHECK: d1 ceildiv 2
        print(expr)

        assert expr.lhs == d1
        assert expr.rhs == c2

        expr2 = AffineExpr.get_ceil_div(c2, d1)
        expr3 = AffineExpr.get_ceil_div(2, d1)
        expr4 = AffineExpr.get_ceil_div(d1, 2)

        # CHECK: 2 ceildiv d1
        print(expr2)
        # CHECK: 2 ceildiv d1
        print(expr3)
        # CHECK: d1 ceildiv 2
        print(expr4)

        assert expr2 == expr3
        assert expr4 == expr


# CHECK-LABEL: TEST: testAffineExprSub
@run
def testAffineExprSub():
    with Context():
        d1 = AffineDimExpr.get(1)
        d2 = AffineDimExpr.get(2)
        expr = d1 - d2
        # CHECK: d1 - d2
        print(expr)

        assert expr.lhs == d1
        rhs = AffineMulExpr(expr.rhs)
        # CHECK: d2
        print(rhs.lhs)
        # CHECK: -1
        print(rhs.rhs)

        # CHECK: d1 - 42
        print(d1 - 42)
        # CHECK: -d1 + 42
        print(42 - d1)

        c42 = AffineConstantExpr.get(42)
        assert d1 - 42 == d1 - c42
        assert 42 - d1 == c42 - d1


# CHECK-LABEL: TEST: testClassHierarchy
@run
def testClassHierarchy():
    with Context():
        d1 = AffineDimExpr.get(1)
        c2 = AffineConstantExpr.get(2)
        add = AffineAddExpr.get(d1, c2)
        mul = AffineMulExpr.get(d1, c2)
        mod = AffineModExpr.get(d1, c2)
        floor_div = AffineFloorDivExpr.get(d1, c2)
        ceil_div = AffineCeilDivExpr.get(d1, c2)

        # CHECK: False
        print(isinstance(d1, AffineBinaryExpr))
        # CHECK: False
        print(isinstance(c2, AffineBinaryExpr))
        # CHECK: True
        print(isinstance(add, AffineBinaryExpr))
        # CHECK: True
        print(isinstance(mul, AffineBinaryExpr))
        # CHECK: True
        print(isinstance(mod, AffineBinaryExpr))
        # CHECK: True
        print(isinstance(floor_div, AffineBinaryExpr))
        # CHECK: True
        print(isinstance(ceil_div, AffineBinaryExpr))

        try:
            AffineBinaryExpr(d1)
        except ValueError as e:
            # CHECK: Cannot cast affine expression to AffineBinaryExpr
            print(e)

        try:
            AffineBinaryExpr(c2)
        except ValueError as e:
            # CHECK: Cannot cast affine expression to AffineBinaryExpr
            print(e)


# CHECK-LABEL: TEST: testIsInstance
@run
def testIsInstance():
    with Context():
        d1 = AffineDimExpr.get(1)
        c2 = AffineConstantExpr.get(2)
        add = AffineAddExpr.get(d1, c2)
        mul = AffineMulExpr.get(d1, c2)

        # CHECK: True
        print(isinstance(d1, AffineDimExpr))
        # CHECK: False
        print(isinstance(d1, AffineConstantExpr))
        # CHECK: True
        print(isinstance(c2, AffineConstantExpr))
        # CHECK: False
        print(isinstance(c2, AffineMulExpr))
        # CHECK: True
        print(isinstance(add, AffineAddExpr))
        # CHECK: False
        print(isinstance(add, AffineMulExpr))
        # CHECK: True
        print(isinstance(mul, AffineMulExpr))
        # CHECK: False
        print(isinstance(mul, AffineAddExpr))


# CHECK-LABEL: TEST: testCompose
@run
def testCompose():
    with Context():
        # d0 + d2.
        expr = AffineAddExpr.get(AffineDimExpr.get(0), AffineDimExpr.get(2))

        # (d0, d1, d2)[s0, s1] -> (d0 + s1, d1 + s0, d0 + d1 + d2)
        map1 = AffineAddExpr.get(AffineDimExpr.get(0), AffineSymbolExpr.get(1))
        map2 = AffineAddExpr.get(AffineDimExpr.get(1), AffineSymbolExpr.get(0))
        map3 = AffineAddExpr.get(
            AffineAddExpr.get(AffineDimExpr.get(0), AffineDimExpr.get(1)),
            AffineDimExpr.get(2),
        )
        map = AffineMap.get(3, 2, [map1, map2, map3])

        # CHECK: d0 + s1 + d0 + d1 + d2
        print(expr.compose(map))


# CHECK-LABEL: TEST: testHash
@run
def testHash():
    with Context():
        d0 = AffineDimExpr.get(0)
        s1 = AffineSymbolExpr.get(1)
        assert hash(d0) == hash(AffineDimExpr.get(0))
        assert hash(d0 + s1) == hash(AffineAddExpr.get(d0, s1))

        dictionary = dict()
        dictionary[d0] = 0
        dictionary[s1] = 1
        assert d0 in dictionary
        assert s1 in dictionary


# CHECK-LABEL: TEST: testAffineExprShift
@run
def testAffineExprShift():
    with Context() as ctx:
        dims = [AffineExpr.get_dim(i) for i in range(4)]
        syms = [AffineExpr.get_symbol(i) for i in range(4)]

        assert (dims[2] + dims[3]) == (dims[0] + dims[1]).shift_dims(2, 2)
        assert (syms[2] + syms[3]) == (syms[0] + syms[1]).shift_symbols(2, 2, 0)


# CHECK-LABEL: TEST: testAffineExprSimplify
@run
def testAffineExprSimplify():
    with Context() as ctx:
        expr = AffineExpr.get_dim(0) + AffineExpr.get_symbol(0)
        assert expr == AffineExpr.simplify_affine_expr(expr, 1, 1)


# CHECK-LABEL: TEST: testAffineExprDowncast
@run
def testAffineExprDowncast():
    with Context() as ctx:
        expr = AffineExpr.get_dim(0) + AffineExpr.get_symbol(0)
        assert isinstance(expr.lhs, AffineDimExpr)
        assert isinstance(expr.rhs, AffineSymbolExpr)
        assert expr.lhs.position == 0
        assert expr.rhs.position == 0



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test(self):
        self.build()
        _, process, _, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.c")
        )
        self.runCmd("command script import thing_formatter.py")
        frame = process.selected_thread.selected_frame
        x = frame.var("x")
        names = ("zero", "one")
        for i in range(x.num_children):
            idx = x.GetIndexOfChildWithName(f"[{i}]")
            self.assertEqual(idx, i)
            child = x.GetChildAtIndex(idx)
            self.assertEqual(child.name, names[idx])
        idx = x.GetIndexOfChildWithName(f"[{x.num_children + 1}]")
        self.assertEqual(idx, lldb.UINT32_MAX)



"""
This is a test to ensure that both lldb is reconstructing the right
calling convention for a CXXRecordDecl as represented by:

   DW_CC_pass_by_reference
   DW_CC_pass_by_value

and to also make sure that the ASTImporter is copying over this
setting when importing the CXXRecordDecl via setArgPassingRestrictions.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestArgumentPassingRestrictions(TestBase):
    @skipIf(compiler="clang", compiler_version=["<", "7.0"])
    def test_argument_passing_restrictions(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        self.expect_expr(
            "returnPassByRef()",
            result_type="PassByRef",
            result_children=[ValueCheck(name="x", value="11223344")],
        )

        self.expect_expr("takePassByRef(p)", result_type="int", result_value="42")



"""
Test lldb-dap locations request
"""


import dap_server
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import lldbdap_testcase
import os


class TestDAP_locations(lldbdap_testcase.DAPTestCaseBase):
    @skipIfWindows
    def test_locations(self):
        """
        Tests the 'locations' request.
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)
        source = "main.cpp"
        self.source_path = os.path.join(os.getcwd(), source)
        self.set_source_breakpoints(
            source,
            [line_number(source, "break here")],
        )
        self.continue_to_next_stop()

        locals = {l["name"]: l for l in self.dap_server.get_local_variables()}

        # var1 has a declarationLocation but no valueLocation
        self.assertIn("declarationLocationReference", locals["var1"].keys())
        self.assertNotIn("valueLocationReference", locals["var1"].keys())
        loc_var1 = self.dap_server.request_locations(
            locals["var1"]["declarationLocationReference"]
        )
        self.assertTrue(loc_var1["success"])
        self.assertTrue(loc_var1["body"]["source"]["path"].endswith("main.cpp"))
        self.assertEqual(loc_var1["body"]["line"], 6)

        # func_ptr has both a declaration and a valueLocation
        self.assertIn("declarationLocationReference", locals["func_ptr"].keys())
        self.assertIn("valueLocationReference", locals["func_ptr"].keys())
        decl_loc_func_ptr = self.dap_server.request_locations(
            locals["func_ptr"]["declarationLocationReference"]
        )
        self.assertTrue(decl_loc_func_ptr["success"])
        self.assertTrue(
            decl_loc_func_ptr["body"]["source"]["path"].endswith("main.cpp")
        )
        self.assertEqual(decl_loc_func_ptr["body"]["line"], 7)
        val_loc_func_ptr = self.dap_server.request_locations(
            locals["func_ptr"]["valueLocationReference"]
        )
        self.assertTrue(val_loc_func_ptr["success"])
        self.assertTrue(val_loc_func_ptr["body"]["source"]["path"].endswith("main.cpp"))
        self.assertEqual(val_loc_func_ptr["body"]["line"], 3)

        # func_ref has both a declaration and a valueLocation
        self.assertIn("declarationLocationReference", locals["func_ref"].keys())
        self.assertIn("valueLocationReference", locals["func_ref"].keys())
        decl_loc_func_ref = self.dap_server.request_locations(
            locals["func_ref"]["declarationLocationReference"]
        )
        self.assertTrue(decl_loc_func_ref["success"])
        self.assertTrue(
            decl_loc_func_ref["body"]["source"]["path"].endswith("main.cpp")
        )
        self.assertEqual(decl_loc_func_ref["body"]["line"], 8)
        val_loc_func_ref = self.dap_server.request_locations(
            locals["func_ref"]["valueLocationReference"]
        )
        self.assertTrue(val_loc_func_ref["success"])
        self.assertTrue(val_loc_func_ref["body"]["source"]["path"].endswith("main.cpp"))
        self.assertEqual(val_loc_func_ref["body"]["line"], 3)

        # `evaluate` responses for function pointers also have locations associated
        eval_res = self.dap_server.request_evaluate("greet")
        self.assertTrue(eval_res["success"])
        self.assertIn("valueLocationReference", eval_res["body"].keys())



# RUN: %PYTHON %s | FileCheck %s

from mlir.dialects import smt, arith
from mlir.ir import Context, Location, Module, InsertionPoint, F32Type


def run(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            f(module)
        print(module)
        assert module.operation.verify()


# CHECK-LABEL: TEST: test_smoke
@run
def test_smoke(_module):
    true = smt.constant(True)
    false = smt.constant(False)
    # CHECK: smt.constant true
    # CHECK: smt.constant false


# CHECK-LABEL: TEST: test_types
@run
def test_types(_module):
    bool_t = smt.bool_t()
    bitvector_t = smt.bv_t(5)
    # CHECK: !smt.bool
    print(bool_t)
    # CHECK: !smt.bv<5>
    print(bitvector_t)


# CHECK-LABEL: TEST: test_solver_op
@run
def test_solver_op(_module):
    @smt.solver
    def foo1():
        true = smt.constant(True)
        false = smt.constant(False)

    # CHECK: smt.solver() : () -> () {
    # CHECK:   %true = smt.constant true
    # CHECK:   %false = smt.constant false
    # CHECK: }

    f32 = F32Type.get()

    @smt.solver(results=[f32])
    def foo2():
        return arith.ConstantOp(f32, 1.0)

    # CHECK: %{{.*}} = smt.solver() : () -> f32 {
    # CHECK:   %[[CST1:.*]] = arith.constant 1.000000e+00 : f32
    # CHECK:   smt.yield %[[CST1]] : f32
    # CHECK: }

    two = arith.ConstantOp(f32, 2.0)
    # CHECK: %[[CST2:.*]] = arith.constant 2.000000e+00 : f32
    print(two)

    @smt.solver(inputs=[two], results=[f32])
    def foo3(z: f32):
        return z

    # CHECK: %{{.*}} = smt.solver(%[[CST2]]) : (f32) -> f32 {
    # CHECK: ^bb0(%[[ARG0:.*]]: f32):
    # CHECK:   smt.yield %[[ARG0]] : f32
    # CHECK: }


# CHECK-LABEL: TEST: test_export_smtlib
@run
def test_export_smtlib(module):
    @smt.solver
    def foo1():
        true = smt.constant(True)
        smt.assert_(true)

    query = smt.export_smtlib(module.operation)
    # CHECK: ; solver scope 0
    # CHECK: (assert true)
    # CHECK: (reset)
    print(query)



"""Test that stepping in object files with multiple compile units works."""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class TestFullLtoStepping(TestBase):
    # The Makefile manually invokes clang.
    @skipIfAsan
    @skipIf(compiler=no_match("clang"))
    @skipIf(compiler="clang", compiler_version=["<", "13.0"])
    @skipUnlessDarwin
    def test(self):
        self.build()
        _, _, thread, _ = lldbutil.run_to_name_breakpoint(self, "main")

        name = thread.frames[0].GetFunctionName()
        # Check that we start out in main.
        self.assertEqual(name, "main")
        thread.StepInto()
        name = thread.frames[0].GetFunctionName()
        # Check that we stepped into f.
        self.assertEqual(name, "f")



#!/usr/bin/env python
# ===-- x86_64_target_definition.py -----------------------------*- C++ -*-===//
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
#
# ===----------------------------------------------------------------------===//

# ----------------------------------------------------------------------
# DESCRIPTION
#
# This file can be used with the following setting:
#   plugin.process.gdb-remote.target-definition-file
# This setting should be used when you are trying to connect to a
# remote GDB server that doesn't support any of the register discovery
# packets that LLDB normally uses.
#
# Why is this necessary? LLDB doesn't require a new build of LLDB that
# targets each new architecture you will debug with. Instead, all
# architectures are supported and LLDB relies on extra GDB server
# packets to discover the target we are connecting to so that is can
# show the right registers for each target. This allows the GDB server
# to change and add new registers without requiring a new LLDB build
# just so we can see new registers.
#
# This file implements the x86_64 registers for the darwin version of
# GDB and allows you to connect to servers that use this register set.
#
# USAGE
#
# (lldb) settings set plugin.process.gdb-remote.target-definition-file /path/to/x86_64_target_definition.py
# (lldb) gdb-remote other.baz.com:1234
#
# The target definition file will get used if and only if the
# qRegisterInfo packets are not supported when connecting to a remote
# GDB server.
# ----------------------------------------------------------------------
from lldb import *

# Compiler and DWARF register numbers
name_to_gcc_dwarf_regnum = {
    "rax": 0,
    "rdx": 1,
    "rcx": 2,
    "rbx": 3,
    "rsi": 4,
    "rdi": 5,
    "rbp": 6,
    "rsp": 7,
    "r8": 8,
    "r9": 9,
    "r10": 10,
    "r11": 11,
    "r12": 12,
    "r13": 13,
    "r14": 14,
    "r15": 15,
    "rip": 16,
    "xmm0": 17,
    "xmm1": 18,
    "xmm2": 19,
    "xmm3": 20,
    "xmm4": 21,
    "xmm5": 22,
    "xmm6": 23,
    "xmm7": 24,
    "xmm8": 25,
    "xmm9": 26,
    "xmm10": 27,
    "xmm11": 28,
    "xmm12": 29,
    "xmm13": 30,
    "xmm14": 31,
    "xmm15": 32,
    "stmm0": 33,
    "stmm1": 34,
    "stmm2": 35,
    "stmm3": 36,
    "stmm4": 37,
    "stmm5": 38,
    "stmm6": 39,
    "stmm7": 30,
    "ymm0": 41,
    "ymm1": 42,
    "ymm2": 43,
    "ymm3": 44,
    "ymm4": 45,
    "ymm5": 46,
    "ymm6": 47,
    "ymm7": 48,
    "ymm8": 49,
    "ymm9": 40,
    "ymm10": 41,
    "ymm11": 42,
    "ymm12": 43,
    "ymm13": 44,
    "ymm14": 45,
    "ymm15": 46,
}

name_to_gdb_regnum = {
    "rax": 0,
    "rbx": 1,
    "rcx": 2,
    "rdx": 3,
    "rsi": 4,
    "rdi": 5,
    "rbp": 6,
    "rsp": 7,
    "r8": 8,
    "r9": 9,
    "r10": 10,
    "r11": 11,
    "r12": 12,
    "r13": 13,
    "r14": 14,
    "r15": 15,
    "rip": 16,
    "rflags": 17,
    "cs": 18,
    "ss": 19,
    "ds": 20,
    "es": 21,
    "fs": 22,
    "gs": 23,
    "stmm0": 24,
    "stmm1": 25,
    "stmm2": 26,
    "stmm3": 27,
    "stmm4": 28,
    "stmm5": 29,
    "stmm6": 30,
    "stmm7": 31,
    "fctrl": 32,
    "fstat": 33,
    "ftag": 34,
    "fiseg": 35,
    "fioff": 36,
    "foseg": 37,
    "fooff": 38,
    "fop": 39,
    "xmm0": 40,
    "xmm1": 41,
    "xmm2": 42,
    "xmm3": 43,
    "xmm4": 44,
    "xmm5": 45,
    "xmm6": 46,
    "xmm7": 47,
    "xmm8": 48,
    "xmm9": 49,
    "xmm10": 50,
    "xmm11": 51,
    "xmm12": 52,
    "xmm13": 53,
    "xmm14": 54,
    "xmm15": 55,
    "mxcsr": 56,
    "ymm0": 57,
    "ymm1": 58,
    "ymm2": 59,
    "ymm3": 60,
    "ymm4": 61,
    "ymm5": 62,
    "ymm6": 63,
    "ymm7": 64,
    "ymm8": 65,
    "ymm9": 66,
    "ymm10": 67,
    "ymm11": 68,
    "ymm12": 69,
    "ymm13": 70,
    "ymm14": 71,
    "ymm15": 72,
}

name_to_generic_regnum = {
    "rip": LLDB_REGNUM_GENERIC_PC,
    "rsp": LLDB_REGNUM_GENERIC_SP,
    "rbp": LLDB_REGNUM_GENERIC_FP,
    "rdi": LLDB_REGNUM_GENERIC_ARG1,
    "rsi": LLDB_REGNUM_GENERIC_ARG2,
    "rdx": LLDB_REGNUM_GENERIC_ARG3,
    "rcx": LLDB_REGNUM_GENERIC_ARG4,
    "r8": LLDB_REGNUM_GENERIC_ARG5,
    "r9": LLDB_REGNUM_GENERIC_ARG6,
}


def get_reg_num(reg_num_dict, reg_name):
    if reg_name in reg_num_dict:
        return reg_num_dict[reg_name]
    return LLDB_INVALID_REGNUM


def get_reg_num(reg_num_dict, reg_name):
    if reg_name in reg_num_dict:
        return reg_num_dict[reg_name]
    return LLDB_INVALID_REGNUM


x86_64_register_infos = [
    {
        "name": "rax",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
    },
    {
        "name": "rbx",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
    },
    {
        "name": "rcx",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
        "alt-name": "arg4",
    },
    {
        "name": "rdx",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
        "alt-name": "arg3",
    },
    {
        "name": "rsi",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
        "alt-name": "arg2",
    },
    {
        "name": "rdi",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
        "alt-name": "arg1",
    },
    {
        "name": "rbp",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
        "alt-name": "fp",
    },
    {
        "name": "rsp",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
        "alt-name": "sp",
    },
    {
        "name": "r8",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
        "alt-name": "arg5",
    },
    {
        "name": "r9",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
        "alt-name": "arg6",
    },
    {
        "name": "r10",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
    },
    {
        "name": "r11",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
    },
    {
        "name": "r12",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
    },
    {
        "name": "r13",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
    },
    {
        "name": "r14",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
    },
    {
        "name": "r15",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
    },
    {
        "name": "rip",
        "set": 0,
        "bitsize": 64,
        "encoding": eEncodingUint,
        "format": eFormatAddressInfo,
        "alt-name": "pc",
    },
    {
        "name": "rflags",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
    },
    {
        "name": "cs",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
    },
    {
        "name": "ss",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
    },
    {
        "name": "ds",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
    },
    {
        "name": "es",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
    },
    {
        "name": "fs",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
    },
    {
        "name": "gs",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
    },
    {
        "name": "stmm0",
        "set": 1,
        "bitsize": 80,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "stmm1",
        "set": 1,
        "bitsize": 80,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "stmm2",
        "set": 1,
        "bitsize": 80,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "stmm3",
        "set": 1,
        "bitsize": 80,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "stmm4",
        "set": 1,
        "bitsize": 80,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "stmm5",
        "set": 1,
        "bitsize": 80,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "stmm6",
        "set": 1,
        "bitsize": 80,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "stmm7",
        "set": 1,
        "bitsize": 80,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "fctrl",
        "set": 1,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
    },
    {
        "name": "fstat",
        "set": 1,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
    },
    {
        "name": "ftag",
        "set": 1,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
    },
    {
        "name": "fiseg",
        "set": 1,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
    },
    {
        "name": "fioff",
        "set": 1,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
    },
    {
        "name": "foseg",
        "set": 1,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
    },
    {
        "name": "fooff",
        "set": 1,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
    },
    {
        "name": "fop",
        "set": 1,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
    },
    {
        "name": "xmm0",
        "set": 1,
        "bitsize": 128,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "xmm1",
        "set": 1,
        "bitsize": 128,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "xmm2",
        "set": 1,
        "bitsize": 128,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "xmm3",
        "set": 1,
        "bitsize": 128,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "xmm4",
        "set": 1,
        "bitsize": 128,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "xmm5",
        "set": 1,
        "bitsize": 128,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "xmm6",
        "set": 1,
        "bitsize": 128,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "xmm7",
        "set": 1,
        "bitsize": 128,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "xmm8",
        "set": 1,
        "bitsize": 128,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "xmm9",
        "set": 1,
        "bitsize": 128,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "xmm10",
        "set": 1,
        "bitsize": 128,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "xmm11",
        "set": 1,
        "bitsize": 128,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "xmm12",
        "set": 1,
        "bitsize": 128,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "xmm13",
        "set": 1,
        "bitsize": 128,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "xmm14",
        "set": 1,
        "bitsize": 128,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "xmm15",
        "set": 1,
        "bitsize": 128,
        "encoding": eEncodingVector,
        "format": eFormatVectorOfUInt8,
    },
    {
        "name": "mxcsr",
        "set": 1,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
    },
    # Registers that are contained in or composed of one of more other
    # registers
    {
        "name": "eax",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rax[31:0]",
    },
    {
        "name": "ebx",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rbx[31:0]",
    },
    {
        "name": "ecx",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rcx[31:0]",
    },
    {
        "name": "edx",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rdx[31:0]",
    },
    {
        "name": "edi",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rdi[31:0]",
    },
    {
        "name": "esi",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rsi[31:0]",
    },
    {
        "name": "ebp",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rbp[31:0]",
    },
    {
        "name": "esp",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rsp[31:0]",
    },
    {
        "name": "r8d",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r8[31:0]",
    },
    {
        "name": "r9d",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r9[31:0]",
    },
    {
        "name": "r10d",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r10[31:0]",
    },
    {
        "name": "r11d",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r11[31:0]",
    },
    {
        "name": "r12d",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r12[31:0]",
    },
    {
        "name": "r13d",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r13[31:0]",
    },
    {
        "name": "r14d",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r14[31:0]",
    },
    {
        "name": "r15d",
        "set": 0,
        "bitsize": 32,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r15[31:0]",
    },
    {
        "name": "ax",
        "set": 0,
        "bitsize": 16,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rax[15:0]",
    },
    {
        "name": "bx",
        "set": 0,
        "bitsize": 16,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rbx[15:0]",
    },
    {
        "name": "cx",
        "set": 0,
        "bitsize": 16,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rcx[15:0]",
    },
    {
        "name": "dx",
        "set": 0,
        "bitsize": 16,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rdx[15:0]",
    },
    {
        "name": "di",
        "set": 0,
        "bitsize": 16,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rdi[15:0]",
    },
    {
        "name": "si",
        "set": 0,
        "bitsize": 16,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rsi[15:0]",
    },
    {
        "name": "bp",
        "set": 0,
        "bitsize": 16,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rbp[15:0]",
    },
    {
        "name": "sp",
        "set": 0,
        "bitsize": 16,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rsp[15:0]",
    },
    {
        "name": "r8w",
        "set": 0,
        "bitsize": 16,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r8[15:0]",
    },
    {
        "name": "r9w",
        "set": 0,
        "bitsize": 16,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r9[15:0]",
    },
    {
        "name": "r10w",
        "set": 0,
        "bitsize": 16,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r10[15:0]",
    },
    {
        "name": "r11w",
        "set": 0,
        "bitsize": 16,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r11[15:0]",
    },
    {
        "name": "r12w",
        "set": 0,
        "bitsize": 16,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r12[15:0]",
    },
    {
        "name": "r13w",
        "set": 0,
        "bitsize": 16,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r13[15:0]",
    },
    {
        "name": "r14w",
        "set": 0,
        "bitsize": 16,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r14[15:0]",
    },
    {
        "name": "r15w",
        "set": 0,
        "bitsize": 16,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r15[15:0]",
    },
    {
        "name": "ah",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rax[15:8]",
    },
    {
        "name": "bh",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rbx[15:8]",
    },
    {
        "name": "ch",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rcx[15:8]",
    },
    {
        "name": "dh",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rdx[15:8]",
    },
    {
        "name": "al",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rax[7:0]",
    },
    {
        "name": "bl",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rbx[7:0]",
    },
    {
        "name": "cl",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rcx[7:0]",
    },
    {
        "name": "dl",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rdx[7:0]",
    },
    {
        "name": "dil",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rdi[7:0]",
    },
    {
        "name": "sil",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rsi[7:0]",
    },
    {
        "name": "bpl",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rbp[7:0]",
    },
    {
        "name": "spl",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "rsp[7:0]",
    },
    {
        "name": "r8l",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r8[7:0]",
    },
    {
        "name": "r9l",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r9[7:0]",
    },
    {
        "name": "r10l",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r10[7:0]",
    },
    {
        "name": "r11l",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r11[7:0]",
    },
    {
        "name": "r12l",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r12[7:0]",
    },
    {
        "name": "r13l",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r13[7:0]",
    },
    {
        "name": "r14l",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r14[7:0]",
    },
    {
        "name": "r15l",
        "set": 0,
        "bitsize": 8,
        "encoding": eEncodingUint,
        "format": eFormatHex,
        "slice": "r15[7:0]",
    },
]

g_target_definition = None


def get_target_definition():
    global g_target_definition
    if g_target_definition is None:
        g_target_definition = {}
        offset = 0
        for reg_info in x86_64_register_infos:
            reg_name = reg_info["name"]

            # Only fill in the offset if there is no 'slice' in the register
            # info
            if "slice" not in reg_info and "composite" not in reg_info:
                reg_info["offset"] = offset
                offset += reg_info["bitsize"] // 8

            # Set the GCC/DWARF register number for this register if it has one
            reg_num = get_reg_num(name_to_gcc_dwarf_regnum, reg_name)
            if reg_num != LLDB_INVALID_REGNUM:
                reg_info["gcc"] = reg_num
                reg_info["dwarf"] = reg_num

            # Set the generic register number for this register if it has one
            reg_num = get_reg_num(name_to_generic_regnum, reg_name)
            if reg_num != LLDB_INVALID_REGNUM:
                reg_info["generic"] = reg_num

            # Set the GDB register number for this register if it has one
            reg_num = get_reg_num(name_to_gdb_regnum, reg_name)
            if reg_num != LLDB_INVALID_REGNUM:
                reg_info["gdb"] = reg_num

        g_target_definition["sets"] = [
            "General Purpose Registers",
            "Floating Point Registers",
        ]
        g_target_definition["registers"] = x86_64_register_infos
        g_target_definition["host-info"] = {
            "triple": "x86_64-apple-macosx",
            "endian": eByteOrderLittle,
        }
        g_target_definition["g-packet-size"] = offset
    return g_target_definition


def get_dynamic_setting(target, setting_name):
    if setting_name == "gdb-server-target-definition":
        return get_target_definition()



"""GDB pretty printers for MLIR types."""

import gdb.printing


class StoragePrinter:
    """Prints bases of a struct and its fields."""

    def __init__(self, val):
        self.val = val

    def children(self):
        for field in self.val.type.fields():
            if field.is_base_class:
                yield "<%s>" % field.name, self.val.cast(field.type)
            else:
                yield field.name, self.val[field.name]

    def to_string(self):
        return "mlir::Storage"


class TupleTypeStoragePrinter(StoragePrinter):
    def children(self):
        for child in StoragePrinter.children(self):
            yield child
        pointer_type = gdb.lookup_type("mlir::Type").pointer()
        elements = (self.val.address + 1).cast(pointer_type)
        for i in range(self.val["numElements"]):
            yield "elements[%u]" % i, elements[i]

    def to_string(self):
        return "mlir::TupleTypeStorage of %u elements" % self.val["numElements"]


class FusedLocationStoragePrinter(StoragePrinter):
    def children(self):
        for child in StoragePrinter.children(self):
            yield child
        pointer_type = gdb.lookup_type("mlir::Location").pointer()
        elements = (self.val.address + 1).cast(pointer_type)
        for i in range(self.val["numLocs"]):
            yield "locs[%u]" % i, elements[i]

    def to_string(self):
        return "mlir::FusedLocationStorage of %u locs" % self.val["numLocs"]


class StorageTypeMap:
    """Maps a TypeID to the corresponding concrete type.

    Types need to be registered by name before the first lookup.
    """

    def __init__(self):
        self.map = None
        self.type_names = []

    def register_type(self, type_name):
        assert not self.map, "register_type called after __getitem__"
        self.type_names += [type_name]

    def _init_map(self):
        """Lazy initialization  of self.map."""
        if self.map:
            return
        self.map = {}
        for type_name in self.type_names:
            concrete_type = gdb.lookup_type(type_name)
            try:
                storage = gdb.parse_and_eval(
                    "&'mlir::detail::TypeIDExported::get<%s>()::instance'" % type_name
                )
            except gdb.error:
                # Skip when TypeID instance cannot be found in current context.
                continue
            if concrete_type and storage:
                self.map[int(storage)] = concrete_type

    def __getitem__(self, type_id):
        self._init_map()
        return self.map.get(int(type_id["storage"]))


storage_type_map = StorageTypeMap()


def get_type_id_printer(val):
    """Returns a printer of the name of a mlir::TypeID."""

    class TypeIdPrinter:
        def __init__(self, string):
            self.string = string

        def to_string(self):
            return self.string

    concrete_type = storage_type_map[val]
    if not concrete_type:
        return None
    return TypeIdPrinter("mlir::TypeID::get<%s>()" % concrete_type)


def get_attr_or_type_printer(val, get_type_id):
    """Returns a printer for mlir::Attribute or mlir::Type."""

    class AttrOrTypePrinter:
        def __init__(self, type_id, impl):
            self.type_id = type_id
            self.impl = impl

        def children(self):
            yield "typeID", self.type_id
            yield "impl", self.impl

        def to_string(self):
            return "cast<%s>" % self.impl.type

    if not val["impl"]:
        return None
    impl = val["impl"].dereference()
    type_id = get_type_id(impl)
    concrete_type = storage_type_map[type_id]
    if not concrete_type:
        return None
    # 3rd template argument of StorageUserBase is the storage type.
    storage_type = concrete_type.fields()[0].type.template_argument(2)
    if not storage_type:
        return None
    return AttrOrTypePrinter(type_id, impl.cast(storage_type))


class ImplPrinter:
    """Printer for an instance with a single 'impl' member pointer."""

    def __init__(self, val):
        self.val = val
        self.impl = val["impl"]

    def children(self):
        if self.impl:
            yield "impl", self.impl.dereference()

    def to_string(self):
        return self.val.type.name


# Printers of types deriving from Attribute::AttrBase or Type::TypeBase.
for name in [
    # mlir/IR/Attributes.h
    "ArrayAttr",
    "DictionaryAttr",
    "FloatAttr",
    "IntegerAttr",
    "IntegerSetAttr",
    "OpaqueAttr",
    "StringAttr",
    "SymbolRefAttr",
    "TypeAttr",
    "UnitAttr",
    "DenseStringElementsAttr",
    "DenseTypedElementsAttr",
    "SparseElementsAttr",
    # mlir/IR/BuiltinTypes.h
    "ComplexType",
    "IndexType",
    "IntegerType",
    "Float16Type",
    "FloatTF32Type",
    "Float32Type",
    "Float64Type",
    "Float80Type",
    "Float128Type",
    "NoneType",
    "VectorType",
    "RankedTensorType",
    "UnrankedTensorType",
    "MemRefType",
    "UnrankedMemRefType",
    "TupleType",
    # mlir/IR/Location.h
    "CallSiteLoc",
    "FileLineColLoc",
    "FusedLoc",
    "NameLoc",
    "OpaqueLoc",
    "UnknownLoc",
]:
    storage_type_map.register_type("mlir::%s" % name)  # Register for upcasting.
storage_type_map.register_type("void")  # Register default.


pp = gdb.printing.RegexpCollectionPrettyPrinter("MLIRSupport")

pp.add_printer("mlir::OperationName", "^mlir::OperationName$", ImplPrinter)
pp.add_printer("mlir::Value", "^mlir::Value$", ImplPrinter)

# Printers for types deriving from AttributeStorage or TypeStorage.
pp.add_printer(
    "mlir::detail::FusedLocationStorage",
    "^mlir::detail::FusedLocationStorage",
    FusedLocationStoragePrinter,
)
pp.add_printer(
    "mlir::detail::TupleTypeStorage",
    "^mlir::detail::TupleTypeStorage$",
    TupleTypeStoragePrinter,
)

pp.add_printer("mlir::TypeID", "^mlir::TypeID$", get_type_id_printer)


def add_attr_or_type_printers(name):
    """Adds printers for mlir::Attribute or mlir::Type and their Storage type."""
    get_type_id = lambda val: val["abstract%s" % name]["typeID"]
    pp.add_printer(
        "mlir::%s" % name,
        "^mlir::%s$" % name,
        lambda val: get_attr_or_type_printer(val, get_type_id),
    )


# Upcasting printers of mlir::Attribute and mlir::Type.
for name in ["Attribute", "Type"]:
    add_attr_or_type_printers(name)

gdb.printing.register_pretty_printer(gdb.current_objfile(), pp)



"""
Tests that we can call functions that have definitions in multiple
CUs in the debug-info (which is the case for functions defined in headers).
The linker will most likely de-duplicate the functiond definitions when linking
the final executable. On Darwin, this will create a debug-map that LLDB will use
to fix up object file addresses to addresses in the linked executable. However,
if we parsed the DIE from the object file whose functiond definition got stripped
by the linker, LLDB needs to ensure it can still resolve the function symbol it
got for it.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestFunctionCallFromObjectFile(TestBase):
    def test_lib1(self):
        self.build()
        lldbutil.run_to_name_breakpoint(self, "lib1_func")

        self.expect_expr("Foo{}.foo()", result_type="int", result_value="15")

    def test_lib2(self):
        self.build()
        lldbutil.run_to_name_breakpoint(self, "lib2_func")

        self.expect_expr("Foo{}.foo()", result_type="int", result_value="15")



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def common_setup(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

    def test_call_on_base(self):
        self.common_setup()
        self.expect_expr("base_with_dtor.foo()", result_type="int", result_value="1")
        self.expect_expr("base_without_dtor.foo()", result_type="int", result_value="2")

    def test_call_on_derived(self):
        self.common_setup()
        self.expect_expr("derived_with_dtor.foo()", result_type="int", result_value="3")
        self.expect_expr(
            "derived_without_dtor.foo()", result_type="int", result_value="4"
        )
        self.expect_expr(
            "derived_with_base_dtor.foo()", result_type="int", result_value="5"
        )
        self.expect_expr(
            "derived_with_dtor_but_no_base_dtor.foo()",
            result_type="int",
            result_value="6",
        )

    def test_call_on_derived_as_base(self):
        self.common_setup()
        self.expect_expr(
            "derived_with_dtor_as_base.foo()", result_type="int", result_value="3"
        )
        self.expect_expr(
            "derived_without_as_base.foo()", result_type="int", result_value="4"
        )
        self.expect_expr(
            "derived_with_base_dtor_as_base.foo()", result_type="int", result_value="5"
        )
        self.expect_expr(
            "derived_with_dtor_but_no_base_dtor_as_base.foo()",
            result_type="int",
            result_value="6",
        )

    def test_call_overloaded(self):
        self.common_setup()
        self.expect(
            "expr derived_with_overload.foo()",
            error=True,
            substrs=["too few arguments to function call, expected 1, have 0"],
        )
        self.expect_expr(
            "derived_with_overload.foo(1)", result_type="int", result_value="7"
        )
        self.expect_expr(
            "derived_with_overload_and_using.foo(1)",
            result_type="int",
            result_value="8",
        )
        # FIXME: It seems the using declaration doesn't import the overload from the base class.
        self.expect(
            "expr derived_with_overload_and_using.foo()",
            error=True,
            substrs=["too few arguments to function call, expected 1, have 0"],
        )



import lldb
import unittest
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestMultiprocess(GDBRemoteTestBase):
    def test_qfThreadInfo(self):
        class MyResponder(MockGDBServerResponder):
            def qfThreadInfo(self):
                return "mp400.10200,p400.10204,p401.10300,p400.10208"

        self.server.responder = MyResponder()
        target = self.dbg.CreateTarget("")
        if self.TraceOn():
            self.runCmd("log enable gdb-remote packets")
            self.addTearDownHook(lambda: self.runCmd("log disable gdb-remote packets"))
        process = self.connect(target)
        self.assertEqual(process.id, 0x400)
        self.assertEqual(
            [process.threads[i].id for i in range(process.num_threads)],
            [0x10200, 0x10204, 0x10208],
        )

    def test_stop_reason(self):
        class MyResponder(MockGDBServerResponder):
            def qfThreadInfo(self):
                return "mp400.10200,p400.10204"

            def cont(self):
                return "S02thread:p400.10200;"

        self.server.responder = MyResponder()
        target = self.dbg.CreateTarget("")
        if self.TraceOn():
            self.runCmd("log enable gdb-remote packets")
            self.addTearDownHook(lambda: self.runCmd("log disable gdb-remote packets"))
        process = self.connect(target)
        process.Continue()
        self.assertEqual(
            process.GetThreadByID(0x10200).stop_reason, lldb.eStopReasonSignal
        )
        self.assertEqual(
            process.GetThreadByID(0x10204).stop_reason, lldb.eStopReasonNone
        )



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
import mlir.dialects.builtin as builtin
import mlir.dialects.func as func
import mlir.dialects.vector as vector


def run(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        f()
    return f


# CHECK-LABEL: TEST: testPrintOp
@run
def testPrintOp():
    module = Module.create()
    with InsertionPoint(module.body):

        @func.FuncOp.from_py_func(VectorType.get((12, 5), F32Type.get()))
        def print_vector(arg):
            return vector.PrintOp(source=arg)

    # CHECK-LABEL: func @print_vector(
    # CHECK-SAME:                     %[[ARG:.*]]: vector<12x5xf32>) {
    #       CHECK:   vector.print %[[ARG]] : vector<12x5xf32>
    #       CHECK:   return
    #       CHECK: }
    print(module)


# CHECK-LABEL: TEST: testTransferReadOp
@run
def testTransferReadOp():
    module = Module.create()
    with InsertionPoint(module.body):
        vector_type = VectorType.get([2, 3], F32Type.get())
        memref_type = MemRefType.get(
            [ShapedType.get_dynamic_size(), ShapedType.get_dynamic_size()],
            F32Type.get(),
        )
        index_type = IndexType.get()
        mask_type = VectorType.get(vector_type.shape, IntegerType.get_signless(1))
        identity_map = AffineMap.get_identity(vector_type.rank)
        identity_map_attr = AffineMapAttr.get(identity_map)
        f = func.FuncOp(
            "transfer_read", ([memref_type, index_type, F32Type.get(), mask_type], [])
        )
        with InsertionPoint(f.add_entry_block()):
            A, zero, padding, mask = f.arguments
            vector.TransferReadOp(
                vector_type,
                A,
                [zero, zero],
                identity_map_attr,
                padding,
                [False, False],
                mask=mask,
            )
            vector.TransferReadOp(
                vector_type, A, [zero, zero], identity_map_attr, padding, [False, False]
            )
            func.ReturnOp([])

    # CHECK: @transfer_read(%[[MEM:.*]]: memref<?x?xf32>, %[[IDX:.*]]: index,
    # CHECK: %[[PAD:.*]]: f32, %[[MASK:.*]]: vector<2x3xi1>)
    # CHECK: vector.transfer_read %[[MEM]][%[[IDX]], %[[IDX]]], %[[PAD]], %[[MASK]]
    # CHECK: vector.transfer_read %[[MEM]][%[[IDX]], %[[IDX]]], %[[PAD]]
    # CHECK-NOT: %[[MASK]]
    print(module)


# CHECK-LABEL: TEST: testBitEnumCombiningKind
@run
def testBitEnumCombiningKind():
    module = Module.create()
    with InsertionPoint(module.body):
        f32 = F32Type.get()
        vector_type = VectorType.get([16], f32)

        @func.FuncOp.from_py_func(vector_type)
        def reduction(arg):
            v = vector.ReductionOp(f32, vector.CombiningKind.ADD, arg)
            return v

    # CHECK: func.func @reduction(%[[VEC:.*]]: vector<16xf32>) -> f32 {
    # CHECK: %0 = vector.reduction <add>, %[[VEC]] : vector<16xf32> into f32
    print(module)



"""
Test that we can get the vm range of the shared cache.
"""

import lldb
import re
import json
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SharedCacheVMRangeTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIfRemote
    @skipUnlessDarwin
    @skipIfOutOfTreeDebugserver  # debugserver returns shared_cache_size
    def test_shared_cache_vm_range(self):
        """Test that the shared cache VM range contains a known libc function"""
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c")
        )

        res = lldb.SBCommandReturnObject()
        ci = self.dbg.GetCommandInterpreter()
        self.assertTrue(ci, VALID_COMMAND_INTERPRETER)

        ci.HandleCommand("process plugin packet send jGetSharedCacheInfo:{}", res)

        # packet: jGetSharedCacheInfo:{}
        # response:
        # {
        #   "shared_cache_base_address": 6572900352,
        #   "shared_cache_uuid": "674DB25A-34B2-3C56-8BD4-7D78005B2F2E",
        #   "no_shared_cache": false,
        #   "shared_cache_private_cache": false,
        #   "shared_cache_path": "/System/Volumes/Preboot/Cryptexes/OS/System/Library/dyld/dyld_shared_cache_arm64e",
        #   "shared_cache_size": 5820792832
        # }

        self.assertTrue("response: " in res.GetOutput())
        response = re.search("response: (.+)", res.GetOutput()).group(1)
        json_response = json.loads(response)
        self.assertTrue("shared_cache_base_address" in json_response)
        self.assertTrue("shared_cache_size" in json_response)
        start = json_response["shared_cache_base_address"]
        end = start + json_response["shared_cache_size"]

        symctx_list = target.FindSymbols("printf", lldb.eSymbolTypeCode)
        self.assertGreater(symctx_list.GetSize(), 0)

        symctx = symctx_list.GetContextAtIndex(0)
        sym = symctx.GetSymbol()
        self.assertTrue(sym.IsValid())
        addr = sym.GetStartAddress()
        printf_load_addr = addr.GetLoadAddress(target)

        self.assertGreater(printf_load_addr, start)
        self.assertLess(printf_load_addr, end)



import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class TestMoveNearest(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.line1 = line_number("foo.h", "// !BR1")
        self.line2 = line_number("foo.h", "// !BR2")
        self.line_between = line_number("main.cpp", "// BR_Between")
        print("BR_Between found at", self.line_between)
        self.line_main = line_number("main.cpp", "// !BR_main")

    def test(self):
        """Test target.move-to-nearest logic"""

        self.build()
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.assertTrue(target, VALID_TARGET)

        lldbutil.run_break_set_by_symbol(self, "main", sym_exact=True)
        environment = self.registerSharedLibrariesWithTarget(target, ["foo"])
        process = target.LaunchSimple(
            None, environment, self.get_process_working_directory()
        )
        self.assertState(process.GetState(), lldb.eStateStopped)

        # Regardless of the -m value the breakpoint should have exactly one
        # location on the foo functions
        self.runCmd("settings set target.move-to-nearest-code true")
        lldbutil.run_break_set_by_file_and_line(
            self, "foo.h", self.line1, loc_exact=True, extra_options="-m 1"
        )
        lldbutil.run_break_set_by_file_and_line(
            self, "foo.h", self.line2, loc_exact=True, extra_options="-m 1"
        )

        self.runCmd("settings set target.move-to-nearest-code false")
        lldbutil.run_break_set_by_file_and_line(
            self, "foo.h", self.line1, loc_exact=True, extra_options="-m 0"
        )
        lldbutil.run_break_set_by_file_and_line(
            self, "foo.h", self.line2, loc_exact=True, extra_options="-m 0"
        )

        # Make sure we set a breakpoint in main with -m 1 for various lines in
        # the function declaration
        # "int"
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line_main - 1, extra_options="-m 1"
        )
        # "main()"
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line_main, extra_options="-m 1"
        )
        # "{"
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line_main + 1, extra_options="-m 1"
        )
        # "return .."
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line_main + 2, extra_options="-m 1"
        )

        # Make sure we don't put move the breakpoint if it is set between two functions:
        lldbutil.run_break_set_by_file_and_line(
            self,
            "main.cpp",
            self.line_between,
            extra_options="-m 1",
            num_expected_locations=0,
        )



"""
Test that frame providers wouldn't cause a hang due to a circular dependency
during its initialization.
"""

import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import TestBase
from lldbsuite.test import lldbutil

class FrameProviderCircularDependencyTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        TestBase.setUp(self)
        self.source = "main.c"

    def launch_and_stop_at_breakpoint(self):
        """Build, launch and stop at the breakpoint in bar(). Returns (target, thread)."""
        self.build()

        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.assertTrue(target, "Target should be valid")

        bkpt = target.BreakpointCreateBySourceRegex(
            "break here", lldb.SBFileSpec(self.source)
        )
        self.assertTrue(bkpt.IsValid(), "Breakpoint should be valid")
        self.assertEqual(bkpt.GetNumLocations(), 1, "Should have 1 breakpoint location")

        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, "Process should be valid")
        self.assertEqual(
            process.GetState(), lldb.eStateStopped, "Process should be stopped"
        )

        thread = process.GetSelectedThread()
        self.assertTrue(thread.IsValid(), "Thread should be valid")

        frame0 = thread.GetFrameAtIndex(0)
        self.assertIn("bar", frame0.GetFunctionName(), "Should be stopped in bar()")

        script_path = os.path.join(self.getSourceDir(), "frame_provider.py")
        self.runCmd("command script import " + script_path)

        return target, thread

    @expectedFailureAll(oslist=["linux"], archs=["arm$"])
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24778")
    def test_circular_dependency_with_function_replacement(self):
        """
        Test the circular dependency fix with a provider that replaces function names.
        """
        target, thread = self.launch_and_stop_at_breakpoint()

        original_frame_count = thread.GetNumFrames()
        self.assertGreaterEqual(
            original_frame_count, 3, "Should have at least 3 frames: bar, foo, main"
        )

        frame_names = [thread.GetFrameAtIndex(i).GetFunctionName() for i in range(3)]
        self.assertEqual(frame_names[0], "bar", "Frame 0 should be bar")
        self.assertEqual(frame_names[1], "foo", "Frame 1 should be foo")
        self.assertEqual(frame_names[2], "main", "Frame 2 should be main")

        # Register the frame provider that accesses input_frames.
        # Before the fix, this registration would trigger the circular dependency:
        # - Thread::GetStackFrameList() creates provider
        # - Provider's get_frame_at_index() accesses input_frames[0]
        # - Calls frame.GetFunctionName() -> ExecutionContextRef::GetFrameSP()
        # - Before fix: Calls Thread::GetStackFrameList() again -> CIRCULAR!
        # - After fix: Uses remembered m_frame_list_wp -> Works!
        error = lldb.SBError()
        provider_id = target.RegisterScriptedFrameProvider(
            "frame_provider.ScriptedFrameObjectProvider",
            lldb.SBStructuredData(),
            error,
        )

        # If we reach here without crashing/hanging, the fix is working!
        self.assertTrue(
            error.Success(),
            f"Should successfully register provider (if this fails, circular dependency!): {error}",
        )
        self.assertNotEqual(provider_id, 0, "Provider ID should be non-zero")

        # Verify the provider is working correctly.
        # Frame count should be unchanged (we're replacing frames, not adding).
        new_frame_count = thread.GetNumFrames()
        self.assertEqual(
            new_frame_count,
            original_frame_count,
            "Frame count should be unchanged (replacement, not addition)",
        )

        # Verify that "bar" was replaced with "baz".
        frame0_new = thread.GetFrameAtIndex(0)
        self.assertIsNotNone(frame0_new, "Frame 0 should exist")
        self.assertEqual(
            frame0_new.GetFunctionName(),
            "baz",
            "Frame 0 function should be replaced: bar -> baz",
        )

        # Verify other frames are unchanged.
        frame1_new = thread.GetFrameAtIndex(1)
        self.assertEqual(
            frame1_new.GetFunctionName(), "foo", "Frame 1 should still be foo"
        )

        frame2_new = thread.GetFrameAtIndex(2)
        self.assertEqual(
            frame2_new.GetFunctionName(), "main", "Frame 2 should still be main"
        )

        # Verify we can call methods on all frames (no circular dependency!).
        for i in range(new_frame_count):
            frame = thread.GetFrameAtIndex(i)
            self.assertIsNotNone(frame, f"Frame {i} should exist")
            # These calls should not trigger circular dependency.
            pc = frame.GetPC()
            self.assertNotEqual(pc, 0, f"Frame {i} should have valid PC")
            func_name = frame.GetFunctionName()
            self.assertIsNotNone(func_name, f"Frame {i} should have function name")

    def test_circular_dependency_handle_command_in_init(self):
        """
        Test that calling HandleCommand('bt') in __init__ doesn't cause
        a circular dependency / deadlock.
        """
        target, thread = self.launch_and_stop_at_breakpoint()

        original_frame_count = thread.GetNumFrames()

        # Register a provider that calls HandleCommand("bt") during __init__.
        # Before the fix, this would deadlock because HandleCommand accesses
        # the thread's stack frames, re-entering Thread::GetStackFrameList().
        error = lldb.SBError()
        provider_id = target.RegisterScriptedFrameProvider(
            "frame_provider.HandleCommandInInitProvider",
            lldb.SBStructuredData(),
            error,
        )

        # If we reach here without hanging, the fix is working.
        self.assertTrue(
            error.Success(),
            f"Should successfully register provider: {error}",
        )
        self.assertNotEqual(provider_id, 0, "Provider ID should be non-zero")

        # Verify the provider passes through all frames unchanged.
        new_frame_count = thread.GetNumFrames()
        self.assertEqual(
            new_frame_count,
            original_frame_count,
            "Frame count should be unchanged (pass-through provider)",
        )

        for i in range(min(new_frame_count, 3)):
            frame = thread.GetFrameAtIndex(i)
            self.assertIsNotNone(frame, f"Frame {i} should exist")
            self.assertIsNotNone(
                frame.GetFunctionName(), f"Frame {i} should have function name"
            )

    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24778")
    def test_circular_dependency_evaluate_expression_in_get_frame(self):
        """
        Test that calling EvaluateExpression in get_frame_at_index doesn't
        cause a circular dependency / deadlock.
        """
        target, thread = self.launch_and_stop_at_breakpoint()

        original_frame_count = thread.GetNumFrames()

        # Register a provider that calls EvaluateExpression("baz()") during
        # get_frame_at_index. Before the fix, this would cause a circular
        # dependency because expression evaluation accesses thread frames.
        error = lldb.SBError()
        provider_id = target.RegisterScriptedFrameProvider(
            "frame_provider.EvaluateExpressionInGetFrameProvider",
            lldb.SBStructuredData(),
            error,
        )

        # If we reach here without hanging, the fix is working.
        self.assertTrue(
            error.Success(),
            f"Should successfully register provider: {error}",
        )
        self.assertNotEqual(provider_id, 0, "Provider ID should be non-zero")

        # Verify the provider passes through all frames unchanged.
        new_frame_count = thread.GetNumFrames()
        self.assertEqual(
            new_frame_count,
            original_frame_count,
            "Frame count should be unchanged (pass-through provider)",
        )

        for i in range(min(new_frame_count, 3)):
            frame = thread.GetFrameAtIndex(i)
            self.assertIsNotNone(frame, f"Frame {i} should exist")
            self.assertIsNotNone(
                frame.GetFunctionName(), f"Frame {i} should have function name"
            )



"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class NSArraySyntheticTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.m", "// Set break point at this line.")

    @skipUnlessDarwin
    def test_rdar11086338_with_run_command(self):
        """Test that NSArray reports its synthetic children properly."""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.m", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type synth clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        # Now check that we are displaying Cocoa classes correctly
        self.expect("frame variable arr", substrs=['@"6 elements"'])
        self.expect("frame variable other_arr", substrs=['@"4 elements"'])
        self.expect("frame variable empty_arr", substrs=['@"0 elements"'])
        self.expect(
            "frame variable arr --ptr-depth 1",
            substrs=[
                '@"6 elements"',
                "[0] = 0x",
                "[1] = 0x",
                "[2] = 0x",
                "[3] = 0x",
                "[4] = 0x",
                "[5] = 0x",
            ],
        )
        self.expect(
            "frame variable other_arr --ptr-depth 1",
            substrs=['@"4 elements"', "[0] = 0x", "[1] = 0x", "[2] = 0x", "[3] = 0x"],
        )
        self.expect("frame variable empty_arr --ptr-depth 1", substrs=['@"0 elements"'])
        self.expect(
            "frame variable arr --ptr-depth 1",
            substrs=[
                '@"6 elements"',
                '@"hello"',
                '@"world"',
                '@"this"',
                '@"is"',
                '@"me"',
                '@"http://www.apple.com',
            ],
        )
        self.expect(
            "frame variable other_arr --ptr-depth 1",
            substrs=['@"4 elements"', "(int)5", '@"a string"', '@"6 elements"'],
        )
        self.expect(
            "frame variable other_arr --ptr-depth 2",
            substrs=[
                '@"4 elements"',
                '@"6 elements" {',
                '@"hello"',
                '@"world"',
                '@"this"',
                '@"is"',
                '@"me"',
                '@"http://www.apple.com',
            ],
        )

        self.assertTrue(
            self.frame().FindVariable("arr").MightHaveChildren(),
            "arr says it does not have children!",
        )
        self.assertTrue(
            self.frame().FindVariable("other_arr").MightHaveChildren(),
            "arr says it does not have children!",
        )
        self.assertFalse(
            self.frame().FindVariable("empty_arr").MightHaveChildren(),
            "arr says it does have children!",
        )



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import arith
import mlir.dialects.async_dialect as async_dialect
import mlir.dialects.async_dialect.passes
from mlir.passmanager import *


def run(f):
    print("\nTEST:", f.__name__)
    f()


# CHECK-LABEL: TEST: testCreateGroupOp
@run
def testCreateGroupOp():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            i32 = IntegerType.get_signless(32)
            group_size = arith.ConstantOp(i32, 4)
            async_dialect.create_group(group_size)
        # CHECK:         %0 = "arith.constant"() <{value = 4 : i32}> : () -> i32
        # CHECK:         %1 = "async.create_group"(%0) : (i32) -> !async.group
        print(module)

def testAsyncPass():
    with Context() as context:
        PassManager.parse("any(async-to-async-runtime)")
    print("SUCCESS")


# CHECK-LABEL: testAsyncPass
#       CHECK: SUCCESS
run(testAsyncPass)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @expectedFailureAll(bugnumber="llvm.org/pr50814", compiler="gcc")
    def test_constructors(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )
        self.expect_expr(
            "ClassWithImplicitCtor().foo()", result_type="int", result_value="1"
        )
        self.expect_expr(
            "ClassWithOneCtor(2).value", result_type="int", result_value="2"
        )
        self.expect_expr(
            "ClassWithMultipleCtor(3).value", result_type="int", result_value="3"
        )
        self.expect_expr(
            "ClassWithMultipleCtor(3, 1).value", result_type="int", result_value="4"
        )

        self.expect_expr(
            "ClassWithDeletedCtor().value", result_type="int", result_value="6"
        )
        self.expect_expr(
            "ClassWithDeletedDefaultCtor(7).value", result_type="int", result_value="7"
        )

        # FIXME: It seems we try to call the non-existent default constructor here which is wrong.
        self.expect(
            "expr ClassWithDefaultedCtor().foo()",
            error=True,
            substrs=["Couldn't look up symbols:"],
        )

        # FIXME: Calling deleted constructors should fail before linking.
        self.expect(
            "expr ClassWithDeletedCtor(1).value",
            error=True,
            substrs=["Couldn't look up symbols:"],
        )
        self.expect(
            "expr ClassWithDeletedDefaultCtor().value",
            error=True,
            substrs=["Couldn't look up symbols:", "function", "optimized out"],
        )

    @skipIfWindows  # Can't find operator new.
    @skipIfLinux  # Fails on some Linux systems with SIGABRT.
    def test_constructors_new(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        self.expect_expr(
            "(new ClassWithOneCtor(1))->value; 1", result_type="int", result_value="1"
        )



"""
Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
See https://llvm.org/LICENSE.txt for license information.
SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

Provides classes used by the test results reporting infrastructure
within the LLDB test suite.


This module provides process-management support for the LLDB test
running infrastructure.
"""

# System imports
import os
import re
import signal
import subprocess
import sys
import threading


class CommunicatorThread(threading.Thread):
    """Provides a thread class that communicates with a subprocess."""

    def __init__(self, process, event, output_file):
        super(CommunicatorThread, self).__init__()
        # Don't let this thread prevent shutdown.
        self.daemon = True
        self.process = process
        self.pid = process.pid
        self.event = event
        self.output_file = output_file
        self.output = None

    def run(self):
        try:
            # Communicate with the child process.
            # This will not complete until the child process terminates.
            self.output = self.process.communicate()
        except Exception as exception:  # pylint: disable=broad-except
            if self.output_file:
                self.output_file.write(
                    "exception while using communicate() for pid: {}\n".format(
                        exception
                    )
                )
        finally:
            # Signal that the thread's run is complete.
            self.event.set()


# Provides a regular expression for matching gtimeout-based durations.
TIMEOUT_REGEX = re.compile(r"(^\d+)([smhd])?$")


def timeout_to_seconds(timeout):
    """Converts timeout/gtimeout timeout values into seconds.

    @param timeout a timeout in the form of xm representing x minutes.

    @return None if timeout is None, or the number of seconds as a float
    if a valid timeout format was specified.
    """
    if timeout is None:
        return None
    else:
        match = TIMEOUT_REGEX.match(timeout)
        if match:
            value = float(match.group(1))
            units = match.group(2)
            if units is None:
                # default is seconds.  No conversion necessary.
                return value
            elif units == "s":
                # Seconds.  No conversion necessary.
                return value
            elif units == "m":
                # Value is in minutes.
                return 60.0 * value
            elif units == "h":
                # Value is in hours.
                return (60.0 * 60.0) * value
            elif units == "d":
                # Value is in days.
                return 24 * (60.0 * 60.0) * value
            else:
                raise Exception("unexpected units value '{}'".format(units))
        else:
            raise Exception("could not parse TIMEOUT spec '{}'".format(timeout))


class ProcessHelper(object):
    r"""Provides an interface for accessing process-related functionality.

    This class provides a factory method that gives the caller a
    platform-specific implementation instance of the class.

    Clients of the class should stick to the methods provided in this
    base class.

    \see ProcessHelper.process_helper()
    """

    def __init__(self):
        super(ProcessHelper, self).__init__()

    @classmethod
    def process_helper(cls):
        """Returns a platform-specific ProcessHelper instance.
        @return a ProcessHelper instance that does the right thing for
        the current platform.
        """

        # If you add a new platform, create an instance here and
        # return it.
        if os.name == "nt":
            return WindowsProcessHelper()
        else:
            # For all POSIX-like systems.
            return UnixProcessHelper()

    def create_piped_process(self, command, new_process_group=True):
        # pylint: disable=no-self-use,unused-argument
        # As expected.  We want derived classes to implement this.
        """Creates a subprocess.Popen-based class with I/O piped to the parent.

        @param command the command line list as would be passed to
        subprocess.Popen().  Use the list form rather than the string form.

        @param new_process_group indicates if the caller wants the
        process to be created in its own process group.  Each OS handles
        this concept differently.  It provides a level of isolation and
        can simplify or enable terminating the process tree properly.

        @return a subprocess.Popen-like object.
        """
        raise Exception("derived class must implement")

    def supports_soft_terminate(self):
        # pylint: disable=no-self-use
        # As expected.  We want derived classes to implement this.
        """Indicates if the platform supports soft termination.

        Soft termination is the concept of a terminate mechanism that
        allows the target process to shut down nicely, but with the
        catch that the process might choose to ignore it.

        Platform supporter note: only mark soft terminate as supported
        if the target process has some way to evade the soft terminate
        request; otherwise, just support the hard terminate method.

        @return True if the platform supports a soft terminate mechanism.
        """
        # By default, we do not support a soft terminate mechanism.
        return False

    def soft_terminate(self, popen_process, log_file=None, want_core=True):
        # pylint: disable=no-self-use,unused-argument
        # As expected.  We want derived classes to implement this.
        """Attempts to terminate the process in a polite way.

        This terminate method is intended to give the child process a
        chance to clean up and exit on its own, possibly with a request
        to drop a core file or equivalent (i.e. [mini-]crashdump, crashlog,
        etc.)  If new_process_group was set in the process creation method
        and the platform supports it, this terminate call will attempt to
        kill the whole process tree rooted in this child process.

        @param popen_process the subprocess.Popen-like object returned
        by one of the process-creation methods of this class.

        @param log_file file-like object used to emit error-related
        logging info.  May be None if no error-related info is desired.

        @param want_core True if the caller would like to get a core
        dump (or the analogous crash report) from the terminated process.
        """
        popen_process.terminate()

    def hard_terminate(self, popen_process, log_file=None):
        # pylint: disable=no-self-use,unused-argument
        # As expected.  We want derived classes to implement this.
        """Attempts to terminate the process immediately.

        This terminate method is intended to kill child process in
        a manner in which the child process has no ability to block,
        and also has no ability to clean up properly.  If new_process_group
        was specified when creating the process, and if the platform
        implementation supports it, this will attempt to kill the
        whole process tree rooted in the child process.

        @param popen_process the subprocess.Popen-like object returned
        by one of the process-creation methods of this class.

        @param log_file file-like object used to emit error-related
        logging info.  May be None if no error-related info is desired.
        """
        popen_process.kill()

    def was_soft_terminate(self, returncode, with_core):
        # pylint: disable=no-self-use,unused-argument
        # As expected.  We want derived classes to implement this.
        """Returns if Popen-like object returncode matches soft terminate.

        @param returncode the returncode from the Popen-like object that
        terminated with a given return code.

        @param with_core indicates whether the returncode should match
        a core-generating return signal.

        @return True when the returncode represents what the system would
        issue when a soft_terminate() with the given with_core arg occurred;
        False otherwise.
        """
        if not self.supports_soft_terminate():
            # If we don't support soft termination on this platform,
            # then this should always be False.
            return False
        else:
            # Once a platform claims to support soft terminate, it
            # needs to be able to identify it by overriding this method.
            raise Exception("platform needs to implement")

    def was_hard_terminate(self, returncode):
        # pylint: disable=no-self-use,unused-argument
        # As expected.  We want derived classes to implement this.
        """Returns if Popen-like object returncode matches that of a hard
        terminate attempt.

        @param returncode the returncode from the Popen-like object that
        terminated with a given return code.

        @return True when the returncode represents what the system would
        issue when a hard_terminate() occurred; False
        otherwise.
        """
        raise Exception("platform needs to implement")

    def soft_terminate_signals(self):
        # pylint: disable=no-self-use
        """Retrieve signal numbers that can be sent to soft terminate.
        @return a list of signal numbers that can be sent to soft terminate
        a process, or None if not applicable.
        """
        return None

    def is_exceptional_exit(self, popen_status):
        """Returns whether the program exit status is exceptional.

        Returns whether the return code from a Popen process is exceptional
        (e.g. signals on POSIX systems).

        Derived classes should override this if they can detect exceptional
        program exit.

        @return True if the given popen_status represents an exceptional
        program exit; False otherwise.
        """
        return False

    def exceptional_exit_details(self, popen_status):
        """Returns the normalized exceptional exit code and a description.

        Given an exceptional exit code, returns the integral value of the
        exception (e.g. signal number for POSIX) and a description (e.g.
        signal name on POSIX) for the result.

        Derived classes should override this if they can detect exceptional
        program exit.

        It is fine to not implement this so long as is_exceptional_exit()
        always returns False.

        @return (normalized exception code, symbolic exception description)
        """
        raise Exception("exception_exit_details() called on unsupported class")


class UnixProcessHelper(ProcessHelper):
    """Provides a ProcessHelper for Unix-like operating systems.

    This implementation supports anything that looks Posix-y
    (e.g. Darwin, Linux, *BSD, etc.)
    """

    def __init__(self):
        super(UnixProcessHelper, self).__init__()

    @classmethod
    def _create_new_process_group(cls):
        """Creates a new process group for the calling process."""
        os.setpgid(os.getpid(), os.getpid())

    def create_piped_process(self, command, new_process_group=True):
        # Determine what to run after the fork but before the exec.
        if new_process_group:
            preexec_func = self._create_new_process_group
        else:
            preexec_func = None

        # Create the process.
        process = subprocess.Popen(
            command,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True,  # Elicits automatic byte -> string decoding in Py3
            close_fds=True,
            preexec_fn=preexec_func,
        )

        # Remember whether we're using process groups for this
        # process.
        process.using_process_groups = new_process_group
        return process

    def supports_soft_terminate(self):
        # POSIX does support a soft terminate via:
        # * SIGTERM (no core requested)
        # * SIGQUIT (core requested if enabled, see ulimit -c)
        return True

    @classmethod
    def _validate_pre_terminate(cls, popen_process, log_file):
        # Validate args.
        if popen_process is None:
            raise ValueError("popen_process is None")

        # Ensure we have something that looks like a valid process.
        if popen_process.pid < 1:
            if log_file:
                log_file.write("skipping soft_terminate(): no process id")
            return False

        # We only do the process liveness check if we're not using
        # process groups.  With process groups, checking if the main
        # inferior process is dead and short circuiting here is no
        # good - children of it in the process group could still be
        # alive, and they should be killed during a timeout.
        if not popen_process.using_process_groups:
            # Don't kill if it's already dead.
            popen_process.poll()
            if popen_process.returncode is not None:
                # It has a returncode.  It has already stopped.
                if log_file:
                    log_file.write(
                        "requested to terminate pid {} but it has already "
                        "terminated, returncode {}".format(
                            popen_process.pid, popen_process.returncode
                        )
                    )
                # Move along...
                return False

        # Good to go.
        return True

    def _kill_with_signal(self, popen_process, log_file, signum):
        # Validate we're ready to terminate this.
        if not self._validate_pre_terminate(popen_process, log_file):
            return

        # Choose kill mechanism based on whether we're targeting
        # a process group or just a process.
        try:
            if popen_process.using_process_groups:
                # if log_file:
                #    log_file.write(
                #        "sending signum {} to process group {} now\n".format(
                #            signum, popen_process.pid))
                os.killpg(popen_process.pid, signum)
            else:
                # if log_file:
                #    log_file.write(
                #        "sending signum {} to process {} now\n".format(
                #            signum, popen_process.pid))
                os.kill(popen_process.pid, signum)
        except OSError as error:
            import errno

            if error.errno == errno.ESRCH:
                # This is okay - failed to find the process.  It may be that
                # that the timeout pre-kill hook eliminated the process.  We'll
                # ignore.
                pass
            else:
                raise

    def soft_terminate(self, popen_process, log_file=None, want_core=True):
        # Choose signal based on desire for core file.
        if want_core:
            # SIGQUIT will generate core by default.  Can be caught.
            signum = signal.SIGQUIT
        else:
            # SIGTERM is the traditional nice way to kill a process.
            # Can be caught, doesn't generate a core.
            signum = signal.SIGTERM

        self._kill_with_signal(popen_process, log_file, signum)

    def hard_terminate(self, popen_process, log_file=None):
        self._kill_with_signal(popen_process, log_file, signal.SIGKILL)

    def was_soft_terminate(self, returncode, with_core):
        if with_core:
            return returncode == -signal.SIGQUIT
        else:
            return returncode == -signal.SIGTERM

    def was_hard_terminate(self, returncode):
        return returncode == -signal.SIGKILL

    def soft_terminate_signals(self):
        return [signal.SIGQUIT, signal.SIGTERM]

    def is_exceptional_exit(self, popen_status):
        return popen_status < 0

    @classmethod
    def _signal_names_by_number(cls):
        return dict(
            (k, v)
            for v, k in reversed(sorted(signal.__dict__.items()))
            if v.startswith("SIG") and not v.startswith("SIG_")
        )

    def exceptional_exit_details(self, popen_status):
        signo = -popen_status
        signal_names_by_number = self._signal_names_by_number()
        signal_name = signal_names_by_number.get(signo, "")
        return (signo, signal_name)


class WindowsProcessHelper(ProcessHelper):
    """Provides a Windows implementation of the ProcessHelper class."""

    def __init__(self):
        super(WindowsProcessHelper, self).__init__()

    def create_piped_process(self, command, new_process_group=True):
        if new_process_group:
            # We need this flag if we want os.kill() to work on the subprocess.
            creation_flags = subprocess.CREATE_NEW_PROCESS_GROUP
        else:
            creation_flags = 0

        return subprocess.Popen(
            command,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True,  # Elicits automatic byte -> string decoding in Py3
            creationflags=creation_flags,
        )

    def was_hard_terminate(self, returncode):
        return returncode != 0


class ProcessDriver(object):
    """Drives a child process, notifies on important events, and can timeout.

    Clients are expected to derive from this class and override the
    on_process_started and on_process_exited methods if they want to
    hook either of those.

    This class supports timing out the child process in a platform-agnostic
    way.  The on_process_exited method is informed if the exit was natural
    or if it was due to a timeout.
    """

    def __init__(self, soft_terminate_timeout=10.0):
        super(ProcessDriver, self).__init__()
        self.process_helper = ProcessHelper.process_helper()
        self.pid = None
        # Create the synchronization event for notifying when the
        # inferior dotest process is complete.
        self.done_event = threading.Event()
        self.io_thread = None
        self.process = None
        # Number of seconds to wait for the soft terminate to
        # wrap up, before moving to more drastic measures.
        # Might want this longer if core dumps are generated and
        # take a long time to write out.
        self.soft_terminate_timeout = soft_terminate_timeout
        # Number of seconds to wait for the hard terminate to
        # wrap up, before giving up on the io thread.  This should
        # be fast.
        self.hard_terminate_timeout = 5.0
        self.returncode = None

    # =============================================
    # Methods for subclasses to override if desired.
    # =============================================

    def on_process_started(self):
        pass

    def on_process_exited(self, command, output, was_timeout, exit_status):
        pass

    def on_timeout_pre_kill(self):
        """Called after the timeout interval elapses but before killing it.

        This method is added to enable derived classes the ability to do
        something to the process prior to it being killed.  For example,
        this would be a good spot to run a program that samples the process
        to see what it was doing (or not doing).

        Do not attempt to reap the process (i.e. use wait()) in this method.
        That will interfere with the kill mechanism and return code processing.
        """

    def write(self, content):
        # pylint: disable=no-self-use
        # Intended - we want derived classes to be able to override
        # this and use any self state they may contain.
        sys.stdout.write(content)

    # ==============================================================
    # Operations used to drive processes.  Clients will want to call
    # one of these.
    # ==============================================================

    def run_command(self, command):
        # Start up the child process and the thread that does the
        # communication pump.
        self._start_process_and_io_thread(command)

        # Wait indefinitely for the child process to finish
        # communicating.  This indicates it has closed stdout/stderr
        # pipes and is done.
        self.io_thread.join()
        self.returncode = self.process.wait()
        if self.returncode is None:
            raise Exception(
                "no exit status available for pid {} after the "
                " inferior dotest.py should have completed".format(self.process.pid)
            )

        # Notify of non-timeout exit.
        self.on_process_exited(command, self.io_thread.output, False, self.returncode)

    def run_command_with_timeout(self, command, timeout, want_core):
        # Figure out how many seconds our timeout description is requesting.
        timeout_seconds = timeout_to_seconds(timeout)

        # Start up the child process and the thread that does the
        # communication pump.
        self._start_process_and_io_thread(command)

        self._wait_with_timeout(timeout_seconds, command, want_core)

    # ================
    # Internal details.
    # ================

    def _start_process_and_io_thread(self, command):
        # Create the process.
        self.process = self.process_helper.create_piped_process(command)
        self.pid = self.process.pid
        self.on_process_started()

        # Ensure the event is cleared that is used for signaling
        # from the communication() thread when communication is
        # complete (i.e. the inferior process has finished).
        self.done_event.clear()

        self.io_thread = CommunicatorThread(self.process, self.done_event, self.write)
        self.io_thread.start()

    def _attempt_soft_kill(self, want_core):
        # The inferior dotest timed out.  Attempt to clean it
        # with a non-drastic method (so it can clean up properly
        # and/or generate a core dump).  Often the OS can't guarantee
        # that the process will really terminate after this.
        self.process_helper.soft_terminate(
            self.process, want_core=want_core, log_file=self
        )

        # Now wait up to a certain timeout period for the io thread
        # to say that the communication ended.  If that wraps up
        # within our soft terminate timeout, we're all done here.
        self.io_thread.join(self.soft_terminate_timeout)
        if not self.io_thread.is_alive():
            # stdout/stderr were closed on the child process side. We
            # should be able to wait and reap the child process here.
            self.returncode = self.process.wait()
            # We terminated, and the done_trying result is n/a
            terminated = True
            done_trying = None
        else:
            self.write(
                "soft kill attempt of process {} timed out "
                "after {} seconds\n".format(
                    self.process.pid, self.soft_terminate_timeout
                )
            )
            terminated = False
            done_trying = False
        return terminated, done_trying

    def _attempt_hard_kill(self):
        # Instruct the process to terminate and really force it to
        # happen.  Don't give the process a chance to ignore.
        self.process_helper.hard_terminate(self.process, log_file=self)

        # Reap the child process.  This should not hang as the
        # hard_kill() mechanism is supposed to really kill it.
        # Improvement option:
        # If this does ever hang, convert to a self.process.poll()
        # loop checking on self.process.returncode until it is not
        # None or the timeout occurs.
        self.returncode = self.process.wait()

        # Wait a few moments for the io thread to finish...
        self.io_thread.join(self.hard_terminate_timeout)
        if self.io_thread.is_alive():
            # ... but this is not critical if it doesn't end for some
            # reason.
            self.write(
                "hard kill of process {} timed out after {} seconds waiting "
                "for the io thread (ignoring)\n".format(
                    self.process.pid, self.hard_terminate_timeout
                )
            )

        # Set if it terminated.  (Set up for optional improvement above).
        terminated = self.returncode is not None
        # Nothing else to try.
        done_trying = True

        return terminated, done_trying

    def _attempt_termination(self, attempt_count, want_core):
        if self.process_helper.supports_soft_terminate():
            # When soft termination is supported, we first try to stop
            # the process with a soft terminate.  Failing that, we try
            # the hard terminate option.
            if attempt_count == 1:
                return self._attempt_soft_kill(want_core)
            elif attempt_count == 2:
                return self._attempt_hard_kill()
            else:
                # We don't have anything else to try.
                terminated = self.returncode is not None
                done_trying = True
                return terminated, done_trying
        else:
            # We only try the hard terminate option when there
            # is no soft terminate available.
            if attempt_count == 1:
                return self._attempt_hard_kill()
            else:
                # We don't have anything else to try.
                terminated = self.returncode is not None
                done_trying = True
                return terminated, done_trying

    def _wait_with_timeout(self, timeout_seconds, command, want_core):
        # Allow up to timeout seconds for the io thread to wrap up.
        # If that completes, the child process should be done.
        completed_normally = self.done_event.wait(timeout_seconds)
        if completed_normally:
            # Reap the child process here.
            self.returncode = self.process.wait()
        else:
            # Allow derived classes to do some work after we detected
            # a timeout but before we touch the timed-out process.
            self.on_timeout_pre_kill()

            # Prepare to stop the process
            process_terminated = completed_normally
            terminate_attempt_count = 0

            # Try as many attempts as we support for trying to shut down
            # the child process if it's not already shut down.
            while not process_terminated:
                terminate_attempt_count += 1
                # Attempt to terminate.
                process_terminated, done_trying = self._attempt_termination(
                    terminate_attempt_count, want_core
                )
                # Check if there's nothing more to try.
                if done_trying:
                    # Break out of our termination attempt loop.
                    break

        # At this point, we're calling it good.  The process
        # finished gracefully, was shut down after one or more
        # attempts, or we failed but gave it our best effort.
        self.on_process_exited(
            command, self.io_thread.output, not completed_normally, self.returncode
        )


def patched_init(self, *args, **kwargs):
    self.original_init(*args, **kwargs)
    # Initialize our condition variable that protects wait()/poll().
    self.wait_condition = threading.Condition()


def patched_wait(self, *args, **kwargs):
    self.wait_condition.acquire()
    try:
        result = self.original_wait(*args, **kwargs)
        # The process finished.  Signal the condition.
        self.wait_condition.notify_all()
        return result
    finally:
        self.wait_condition.release()


def patched_poll(self, *args, **kwargs):
    self.wait_condition.acquire()
    try:
        result = self.original_poll(*args, **kwargs)
        if self.returncode is not None:
            # We did complete, and we have the return value.
            # Signal the event to indicate we're done.
            self.wait_condition.notify_all()
        return result
    finally:
        self.wait_condition.release()


def patch_up_subprocess_popen():
    subprocess.Popen.original_init = subprocess.Popen.__init__
    subprocess.Popen.__init__ = patched_init

    subprocess.Popen.original_wait = subprocess.Popen.wait
    subprocess.Popen.wait = patched_wait

    subprocess.Popen.original_poll = subprocess.Popen.poll
    subprocess.Popen.poll = patched_poll


# Replace key subprocess.Popen() threading-unprotected methods with
# threading-protected versions.
patch_up_subprocess_popen()



"""
Tests basic Main Thread Checker support (detecting a main-thread-only violation).
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbplatformutil import *
import json


class MTCSimpleTestCase(TestBase):
    @skipUnlessDarwin
    @skipIf(compiler="clang", compiler_version=["<", "9.0"])
    def test(self):
        self.mtc_dylib_path = findMainThreadCheckerDylib()
        if self.mtc_dylib_path == "":
            self.skipTest("This test requires libMainThreadChecker.dylib")

        self.build()
        self.mtc_tests()

    @skipIf(archs=["i386"])
    @skipIf(compiler="clang", compiler_version=["<", "9.0"])
    def mtc_tests(self):
        self.assertNotEqual(self.mtc_dylib_path, "")

        # Load the test
        exe = self.getBuildArtifact("a.out")
        self.expect("file " + exe, patterns=["Current executable set to .*a.out"])

        self.runCmd("env DYLD_INSERT_LIBRARIES=%s" % self.mtc_dylib_path)
        self.runCmd("run")

        process = self.dbg.GetSelectedTarget().process
        thread = process.GetSelectedThread()
        frame = thread.GetSelectedFrame()

        view = "NSView" if lldbplatformutil.getPlatform() == "macosx" else "UIView"

        self.expect(
            "thread info",
            substrs=[
                "stop reason = -["
                + view
                + " superview] must be used from main thread only"
            ],
        )

        self.expect(
            "thread info -s",
            substrs=[
                "api_name",
                "class_name",
                "description",
                "instrumentation_class",
                "selector",
            ],
        )
        self.assertStopReason(thread.GetStopReason(), lldb.eStopReasonInstrumentation)
        output_lines = self.res.GetOutput().split("\n")
        json_line = "\n".join(output_lines[2:])
        data = json.loads(json_line)
        self.assertEqual(data["instrumentation_class"], "MainThreadChecker")
        self.assertEqual(data["api_name"], "-[" + view + " superview]")
        self.assertEqual(data["class_name"], view)
        self.assertEqual(data["selector"], "superview")
        self.assertEqual(
            data["description"],
            "-[" + view + " superview] must be used from main thread only",
        )
